/*********************************************************************
SortSeries.c 

Read in from the file "normaltoclean" the normalized series with
repetitions removed.  Now we want to put them in lexicographical
order according to the primitive shift vector.  

Write output into "SortedSeries"
  
WARNING!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
For this to run correctly, you must know how long "normaltoclean"
is.  Adjust the matrix dimensions for M and F and the variable
"filelen" accordingly.  M[filelen][11]
                        F[filelen+1][11]
                        filelen = number of lines in "normaltoclean"
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
**********************************************************************/

#include<stdio.h>
#include<stdlib.h>

main()
{
   int a[5];
   int ps[5]; 
   int sm[5];
                      /* the matrices M and F should have filelen rows             */ 
   int M[368][11];    /* the original list of 10-tuples read in from normaltoclean */
   int F[369][11];    /* the final list, in order */

   int found;
   int allcopied;
   int swap;
   int copy;
   int limit;
   int smrow;
   int filelen;   /* how many five-tuples are in normaltoclean */

   int end;
   int i, j, k;
   

   FILE *fp;
   FILE *nfp;
   
  
  /* read in the first 10tuple from the file */
  
   fp = fopen("normaltoclean", "r");
  nfp = fopen("SortedSeries","w");
 
   filelen = 368;

   for(k=0;k<filelen;k++){   /* initialize M and F to be all zeros */
     for(j=0;j<11;j++){
        F[k][j] = 0;
        M[k][j] = 0;
     }
   }

  /* Read in the first row of the file normaltoclean */

  end=fscanf( fp, "%d %d %d %d %d %d %d %d %d %d", 
                 &a[0],&a[1],&a[2],&a[3],&a[4],&ps[0],&ps[1],&ps[2],&ps[3],&ps[4]);
  
  k=0;
  for(i=0; i<5; i++){
    M[k][i] = a[i];
    M[k][i+5] = ps[i];
    sm[i] = ps[i];
  }


/* Now loop through the data to find the smallest shift vector that occurs */ 
  
  while(k < filelen-1 ){  /* changed from 231 */

     end=fscanf( fp, "%d %d %d %d %d %d %d %d %d %d", 
                 &a[0],&a[1],&a[2],&a[3],&a[4],&ps[0],&ps[1],&ps[2],&ps[3],&ps[4]);
     k++;

     for(i=0; i<5; i++){
       M[k][i] = a[i];
       M[k][i+5] = ps[i];
     }

      swap = 0;   /* assume that the first shift vector is the smallest */
                  /* look through the rest and compare to find the smallest shift */

     if(ps[1] < sm[1])
       swap = 1;   /* We found a new candidate for smallest ps */
     if(ps[1]==sm[1] && ps[2] < sm[2])
        swap = 1;
     if(ps[1]==sm[1] && ps[2] == sm[2] && ps[3] < sm[3])
        swap = 1;
     if(ps[1]==sm[1] && ps[2] == sm[2] && ps[3] == sm[3] && ps[4] < sm[4])
        swap = 1;

     
     /* Now we have compared the shift vector in row k with sm */
     if(swap == 1){
       smrow = k;
       for(i=0;i<5; i++)
          sm[i] = ps[i];
     }    
  } /* end of while loop to find the smallest value of ps and read in the data */
  fclose(fp);  /* now we're done with this file */
  

 allcopied = 0;
 j = 0;

 while(allcopied == 0  && j < filelen){

    /* Loop through M.  Whenever we find a row where the shift vector
         matches the current value of sm we copy that row into the next
         row of F.  We then mark that row as copied from M by changing
         M[k][10] from 0 to 1.                                      */
  
    
    for(k=0; k<filelen; k++){
      if(M[k][6] == sm[1] && M[k][7] == sm[2] ){
      if(M[k][8] == sm[3] && M[k][9] == sm[4]){ 
	  for(i=0;i<10;i++){
             F[j][i] = M[k][i];
          }
          j++;
          M[k][10] = 1;   /* mark row k of M as copied */
      }
      }/* end of if there is a match in the shift vector */
    } /* end of loop through M */
 
   
   /* Now we need to find the next smallest shift vector and repeat. */
   /* First loop through M to find a row that has not been copied yet. */

  k = 0;
  found = 0; 
  while(k < filelen && found ==0){
       if(M[k][10]==0){
          found = 1;
          /* replace sm by M[k][6]...M[k][9]  */
          for(i=1;i<5;i++)
               sm[i] = M[k][i+5];
       }
       else
          k++;
 } /* end of while loop to find a row that was not yet copied */

 if(k==filelen) /* all the rows of M are already marked as copied */
     allcopied = 1;
  else{
   /* loop through M again.  if the current row was not yet copied
       then compare to sm and swap if necessary to find the smallest
       shift vector that has not yet been copied.    Otherwise, if the
       current row was already copied just move on.                           */
   
      for(k=0;k<filelen;k++){
           swap = 0;  /* assume that sm already is the shift we want */
           if(M[k][10]==0){ /* if this row has not been copied */
                if(M[k][6] < sm[1])
                    swap = 1;   /* We found a new candidate for smallest ps */
                if(M[k][6]==sm[1] && M[k][7] < sm[2])
                    swap = 1;
                if(M[k][6]==sm[1] && M[k][7] == sm[2] && M[k][8] < sm[3])
                    swap = 1;
                if(M[k][6]==sm[1] && M[k][7] == sm[2] && M[k][8] == sm[3] && M[k][9] < sm[4])
                    swap = 1;
           }/* end of if this row was not copied (if m[k][10 = 0) */

	  if(swap == 1){
            smrow = k;
            for(i=1;i<5; i++)
               sm[i] = M[k][i+5];
          }
          
  } /* end of (k loop) search for new value of sm */
  } /* end of else we need to search for a new value of sm */
 } /* end of while(allcopied == 0) */
 

/* loop to print F */
   j = 0;   
  
   while(j < filelen){
       for(i=0;i<10;i++)
         fprintf(nfp,"%3d ", F[j][i]);
       fprintf(nfp,"\n");
       j++;
   }
   printf("j is %3d\n", j);

fclose(nfp);

} /* matches main */













