/***********************************************************************
                 
alldets.c (July 5, 2000) 

We want to study the problem of solving the system below, later 
referred to as system (1):

   [ m0 + u00    u01       u02         u03      u04] [a0]      [-1] 
   [  u10      m1 + u11    u12         u13      u14] [a1]  =   [-1]
   [  u20        u21      m2 + u22     u23      u24] [a2]      [-1]
   [  u30        u31       u32      m3 + u33    u34] [a3]      [-1]
   [  u40        u41       u42         u43   m4+u44] [a4]      [-1]  
Most of the uij are equal to -1, but in each row we choose
one uij = 0.    
Further restrictions:  m0 is unknown
                       m1 is unknown      (compare to easycramer.c)
                       m2 = 3,4,...,16
                       m3 = 2,3,...,6
                       m4 = 1,2,3

The coefficient matrix of system (1) will be denoted as C in the
comments below.  As usual, we omit the unrealistic case where
C[4][4] = 0.

The integers m0 and m1 are unknown.  The determinant of C is 
a quadratic function of m0 and m1 whose coefficients we compute below:

   detC = c2*m0*m1 + c0*m0 + c1*m1 + c3

If it happens that c2 = 0, and that either c0 or c1 is also 0
then we want to compute all the Cramer's Rule determinants.
Results are written to the file "OtherDets".

***************************************************************************/

#include<stdio.h>
#include<stdlib.h>

main()
{
 int i,j;
 int M[5][5];        /* diagonal matrix with m_i = M[i][i]   */
 int m[5];    
 int mult[2];
 
 int U[5][5];            /* matrix whose entries are mostly -1   */
 int z0, z1, z2, z3, z4; /* where to put 0 in ith row of U       */
                         /* if j(i) = k then zi = k              */
 int C[5][5];        /* the coefficient matrix of system (1) */
 int R[5][5];        /* a copy of C for row reduction        */
 int nocondition;    /* if det C does not depend on m0       */

 int D,c0,c1,c2,c3,a0,b0,a1,b1;         /* det C              */
 int k;
 int d0, d1, d2, d3, d4; /* det's from Cramer's rule             */
 int S[4][4];            /* a 4x4 matrix, usually a subdet of di */
 int T[4][4];            /* a 4x4 matrix for testing */
 int s[3][3];
 int signchange;     /* did we switch two rows in reduction? */
 
 int GiveUp;         /* Is there a problem with this 4tuple? */
 

 int A[5];           /* candidate 5-tuple                    */
 int B[5];           /* same as A, but throw away signs      */

 int t1, t2, t3, t4, t5; /* gcd of three of the A[i]'s           */

 int absb, L;        /* how far do we let m0 run?            */

 int temp;           /* used for sorting --- write the ai's  */
                     /* in increasing order                  */

 int dets[3][4];     /* row 0 holds d2, row 1 holds d3, row 3 holds d4 */
 float fdets[3][4];
 float alpha,beta,gamma;
 int count;

  int gcd2(int a, int b){
      int r;
     
      if (a > b){
      r = a;
      a = b;
      b = r;
      }

      r = b%a;
      while (r > 0){
          b = a;
          a = r;
          r = b%a;
      }
      return a;
  }

  int gcd3(int a, int b, int c){
      return gcd2( gcd2(a,b), c);
  }

int det3( int S[3][3] ){
     int s1,s2,s3;
     s1 = S[0][0]*(S[1][1]*S[2][2] - S[1][2]*S[2][1]);
     s2 = S[1][0]*(S[0][2]*S[2][1] - S[0][1]*S[2][2]);
     s3 = S[2][0]*(S[0][1]*S[1][2] - S[0][2]*S[1][1]);
     
     return s1+s2+s3;
 }

 int det4(int S[4][4]){
     int i,j;
     int d;
     int temp[3][3];

     for(i=1;i<4;i++){              /* let temp hold rows 1 to 3 */
       for(j=1;j<4;j++){            /* and col1,col2,col3 of the */
         temp[i-1][j-1] = S[i][j];  /* 4x4 matrix S              */
       }
     }
     d = S[0][0]*det3(temp);

     for(i=1;i<4;i++)               /* Now we want col0,col2,col3 */
        temp[i-1][0] = S[i][0];
     d = d - S[0][1]*det3(temp);
 
     for(i=1;i<4;i++)               /* Now col0,col1,col3         */
        temp[i-1][1] = S[i][1];
     d = d +S[0][2]*det3(temp);

     for(i=1;i<4;i++)               /* and now col0,col1,col2    */
        temp[i-1][2] = S[i][2];
     d = d - S[0][3]*det3(temp);

     return d;
 }

FILE *fp;
FILE *mfp;
FILE *nfp;
FILE *kfp;


/******************************************
 Initialize the matrix M:
*******************************************/

 for(i = 0; i<5; i++){
   for(j=0; j<5; j++){
     M[i][j] = 0;
   }
 }

 count = 0;
 /*********************************************************
  We have a coefficient matrix C with two unknown entries,
  m0 and m1.  We need expressions for the determinant of 
  C as a function of these unknowns.  We need two other
  determinants as well.  Replace col 0 of C with -1's and
  take the determinant.  Call this d0 --- it is a linear 
  function of m1.  Finally, replace col 1 of the original
  matrix C by -1's and take this determinant.  Call it 
  d1 --- it is a linear function of m0.
  ********************************************************/

 fp = fopen("otherdets", "w");
 
 /**************************************
  Let m[4] run from 1 to 3
  Let m[3] run from 2 to 6
  Let m[2] run from 3 to 16.
  In each case m[i] should equal M[i,i]
 **************************************/
 for(m[4]=1;m[4]<4;m[4]++){ /* should run < 4 in final version */
   M[4][4] = m[4];
   for (m[3]=2; m[3]<7; m[3]++){ /* should run < 7 in final version */
    M[3][3] = m[3];
    for(m[2]=3; m[2]<17; m[2]++){ /* should run < 17 in final version */
      M[2][2]=m[2];
      


       /* at this point we have chosen a matrix M  */
       /* now we need to define the entries of U   */
       /* Start by defining U to have all entries  */
       /* equal to -1.  Then choose one entry in   */
       /* each row to be 0.                        */  

       for(z0=0;z0<5; z0++){
         for(z1=0;z1<5; z1++){
           for(z2=0; z2<5; z2++){
             for(z3=0; z3<5; z3++){
               for(z4=0; z4<5; z4++){
               for(i=0; i<5; i++){ 
                  for(j=0; j<5; j++) {
                      U[i][j] = -1; 
                  }
               }  /* Reinitialize U to be all -1's */

               U[0][z0] = 0;  /* put a 0 in the 0th row */
               U[1][z1] = 0;  /*                1st     */
               U[2][z2] = 0;  /*                2nd     */
               U[3][z3] = 0;  /*                3rd     */
               U[4][z4] = 0;  /* put a 0 in the 4th row */

               /*****************************************
              Now the coefficient matrix C of the system
              (1) can be formed by adding M and U.
              Save a copy as R.  Keep in mind that the 0,0
              entry and the 1,1 entry do not reflect the
              unknown values for m0 and m1.
	     *********************************************/            

              for(i=0; i<5; i++){
                for(j=0; j < 5; j++){
                    C[i][j] = M[i][j] + U[i][j];
                    R[i][j] = C[i][j]; 
                }
              }
	      
              if(C[4][4]!=0){ /* can't occur in a realistic
                                 solution to (1) so exclude */
              
                /*****************************************************
                  Now we compute the determinant of the coefficient
                  matrix C.  It will be of the form
                   c2*m0*m1 + c0*m0 + c1*m1 + c3.
		**************************************************/
		c0 = 0;                   /* Initialize the c's */
                c1 = 0;
                c2 = 0;
                c3 = 0;
                for(i=0;i<5;i++)          /* Reinitialize R */
                  for(j=0;j<5;j++)
                    R[i][j] = C[i][j];

                /* Compute c2, the coefficient of m0,m1. This should
                   be simply the determinant of the 3x3 matrix made
                   from rows 2,3,4 and cols 2,3,4 of R.              */
             
                for(i=2; i<5; i++)
                   for(j=2;j<5;j++)
                      s[i-2][j-2] = R[i][j];
                c2 = det3(s);
                

                 
              /******************************************
               To compute the constant term c3 we simply 
               set m0 = 0 and m1 = 0 and compute det C.

                To compute the linear terms of det C:
                If we set m0 = 0 and compute the determinant
                we get c1*m1 + c3.  If we set m1 = 0 and
                compute det C, then we get c0*m0 + c3.
                Since I already have the constant term all
                c0 will just be the determinant of the 
                4x4 matrix obtained by deleting row 0 and
                col 0 of C.  Similarly, the coefficient 
                c1 of m1 will just be the determinant of
                the 4x4 matrix obtained by deleting row 1
                and column 1 of R.
               *****************************************/
                for(i=1;i<5; i++)
                   for(j=1; j<5; j++)
                     S[i-1][j-1] = R[i][j];
                c3 = c3 + R[0][0]*det4(S); /* compute 0,0 minor of det C */
                c0 = det4(S);

             /********************************************* 
               Now as j runs from 0 to 3 we replace
               column j of S by column j (rows 1,2,3,4)
               of S.  This will give all the minors for
               expanding det C along row 0 
	     ***********************************************/
                    signchange = 1;
		    for(j=0; j<4; j++){
                       signchange = -1*signchange;
                       for(i=0;i<4;i++)
                          S[i][j] = R[i+1][j];
                       c3 = c3 + signchange*R[0][j+1]*det4(S);
                    }
                

	    /****************************************
              All that remains in c1.  Pull out the
              4x4 matrix  obtained by deleting col1 and
              row 1 of R and take the determinant.
            *****************************************/
   
                    S[0][0] = R[0][0];
                    for(j=1;j<4;j++){
                       S[0][j] = R[0][j+1];
                       S[j][0] = R[j+1][0];
                    }
                    for(i=1;i<4;i++)
                       for(j=1;j<4;j++)
                          S[i][j] = R[i+1][j+1];

                    c1 = det4(S);
		   
if(c2==0 && c1*c0 == 0){
     count++;
     /* Reinitialize R and replace column k by -1's
        for k = 2, 3, 4                            */

     /* Reinitialize the array dets                */

     for(i=0; i<3; i++)
        for(j=0;j<4;j++)
           dets[i][j] = 0;
	
     for(k = 2; k < 5; k++){

        for(i=0;i<5;i++)          /* Reinitialize R */
          for(j=0;j<5;j++)
            R[i][j] = C[i][j];
	
        for(i=0;i<5;i++)
           R[i][k] = -1;
        
        /********************************************************* 
           Now for k = 2 we want to compute dets[0][0,1,2,3]
                   k = 3                    dets[1][0,1,2,3]
                   k = 4                    dets[2][0,1,2,3]
           dets[0][2] = coefficient of m0*m1  in d2.
           dets[0][1] = coefficient of m1     in d2.
           dets[0][0] = coefficient of m0     in d2
           dets[0][3] = constant term 
	**********************************************************/
      
        /* Now compute the coefficient of m0*m1. This should
           be simply the determinant of the 3x3 matrix made
           from rows 2,3,4 and cols 2,3,4 of R.              */
             
           for(i=2; i<5; i++)
              for(j=2;j<5;j++)
                 s[i-2][j-2] = R[i][j];
              dets[k-2][2] = det3(s);
                
 
 /******************************************
  To compute the constant term dets[k-2][3] we simply 
  set m0 = 0 and m1 = 0 and compute det R.

  To compute the linear terms of d2,d3,d4:
  If we set m0 = 0 and compute the determinant of R
  we get c1*m1 + c3.  If we set m1 = 0 and
  compute det R, then we get c0*m0 + c3.
  Since I already have the constant term all
  c0 will just be the determinant of the 
  4x4 matrix obtained by deleting row 0 and
  col 0 of R.  Similarly, the coefficient 
  c1 of m1 will just be the determinant of
  the 4x4 matrix obtained by deleting row 1
  and column 1 of R.
  *****************************************/
     for(i=1;i<5; i++)
       for(j=1; j<5; j++)
          S[i-1][j-1] = R[i][j];
       dets[k-2][3] = dets[k-2][3] + R[0][0]*det4(S); 
                                /* compute 0,0 minor of det R */
       dets[k-2][0] = det4(S);

  /********************************************* 
   Now as j runs from 0 to 3 we replace
   column j of S by column j (rows 1,2,3,4)
   of S.  This will give all the minors for
   expanding det R along row 0 
 ***********************************************/
     signchange = 1;
     for(j=0; j<4; j++){
        signchange = -1*signchange;
        for(i=0;i<4;i++)
        S[i][j] = R[i+1][j];
        dets[k-2][3] = dets[k-2][3] + signchange*R[0][j+1]*det4(S);
     }
                

     /********************************************************
     All that remains in dets[k-2][1].  Pull out the
     4x4 matrix  obtained by deleting col1 and
     row 1 of R and take the determinant.
     ********************************************************/
   
        S[0][0] = R[0][0];
        for(j=1;j<4;j++){
           S[0][j] = R[0][j+1];
           S[j][0] = R[j+1][0];
        }
        for(i=1;i<4;i++)
           for(j=1;j<4;j++)
               S[i][j] = R[i+1][j+1];

        dets[k-2][1] = det4(S);    
 
     } /* end of computing d2,d3,d4 (k = 2,3,4) */

     /*  Now we would like to know if d2,d3,d4 are multiples of
     each other --- the hope is that for some i,j = 1,2 or 3 
     we get i*d2 = j*d3, etc.                                  */
  
     for(i=0;i<3;i++)
        for(j=0;j<4; j++)
           fdets[i][j] = dets[i][j];

     for(i=0;i<2;i++){
	mult[i] = 1; /* assume that row i is a multiple of row i+1 */

        /* if any of the 2x2 dets below are not zero then row i is
           not a multiple of row i+1 and we need to adjust.        */

	for(j=0; j<3; j++){
          if(dets[i][j]*dets[i+1][j+1] - dets[i][j+1]*dets[i+1][j]!=0)
            mult[i] = 0;
           }
	}
      
     if(mult[1]*mult[0] != 1) /* d2 and d3 are not multiples, or d3 and d4 are not */
         printf("Watchout\n");
     alpha = 0;
     beta = 0;
     gamma = 0;

     for(j = 0; j<3; j++){
       if(fdets[0][j]*fdets[1][j] !=0)
             alpha = fdets[0][j]/fdets[1][j];
       if(fdets[0][j]*fdets[2][j] != 0)
             beta = fdets[0][j]/fdets[2][j];
       if(fdets[1][j]*fdets[2][j] != 0)
             gamma = fdets[1][j]/fdets[2][j];
     }
     fprintf(fp,"Example %3d: ",count);
     fprintf(fp,"d2/d3 = %3f, d3/d4 = %3f, d2/d4 = %3f\n\n", alpha,gamma,beta);

     
   
     /*********************************************************

            Now print to the file "otherdets"

     fprintf(fp,"**************************************************\n");
     fprintf(fp,"Example %3d: \n", count);
     for(i=0;i<5;i++){
       for(j=0;j<5;j++)
         fprintf(fp, "%3d ", C[i][j]);
       fprintf(fp,"\n");
     }
  

     for(k = 0; k<3; k++)
       fprintf(fp, "d_%d = %3d*m0*m1 + %3d*m0 + %3d*m1 + %3d\n",
                    k+2,dets[k][2],dets[k][0],dets[k][1],dets[k][3]);
       fprintf(fp,"\n");
     fprintf(fp,"\n**************************************************\n");

     ***********************************************************/
           
} /* if c2 = 0 and c1*c0 = 0 */
                                     
	      } /* end of if C[4][4] is not zero */
	       }/* end of z4 loop */
	     }/* end of z3 loop */
           }/* end of z2 loop */
	 } /* end of z1 loop */
       }/* end of z0 loop */
	

   } /* end of m2 loop */
  } /* end of m3 loop */
 } /* end of m4 loop */
            
	       
fclose(fp);

} /* end of main */    



















