/***********************************************************************
                 calabiyau.c

   We want to study the problem of solving the system
     [ m0 + u00     u01           u02             u03] [a0]      [0] 
     [  u10        m1 + u11       u12             u13] [a1]  =   [0]
     [  u20             u21       m2 + u22        u23] [a2]      [0]
     [  u30             u31       u32        m3 + u33] [a3]      [0]
Most of the uij are equal to -1, but in each row we choose
one uij = 0.    Further restrictions:  m1 = 2, 3, ... 11
                                       m2 = 2, 3, 4, 5
                                       m3 =  1, 2
                                       m0 is unknown.
All the numbers a0, a1, a2 and a3 are to be positive integers in
increasing order a0 <= a1 <= a2 <= a3.  We list only primitive
solutions.

***************************************************************************/

#include<stdio.h>
#include<stdlib.h>

main()
{

int A[4];
int B[4];
int M[4][4];
int C[4][4];
int R[4][4];
int U[4][4];
int S[3][3];
int t1, t2, t3, t4;
int D, d0, d1, d2, d3, d4;
int z0, z1, z2, z3;
int m[4];
int i,j;
int a, b;
 int temp, nogood, allzero;

 int gcd2(int a, int b){
      int r;
     
      if (a > b){
      r = a;
      a = b;
      b = r;
      }

      r = b%a;
      while (r > 0){
          b = a;
          a = r;
          r = b%a;
      }
      return a;
  }

 int det3(int M[3][3]){
   int d,m1,m2,m3;
 
   m1 = M[1][1]*M[2][2] - M[1][2]*M[2][1];
   m2 = M[1][2]*M[2][0] - M[1][0]*M[2][2];
   m3 = M[1][0]*M[2][1] - M[1][1]*M[2][0];
  
   d = M[0][0]*m1 + M[0][1]*m2 + M[0][2]*m3;
 
   return d;
 }
FILE *fp;



/******************************************
 Initialize the matrix M and U:
*******************************************/

 for(i = 0; i<4; i++){
   for(j=0; j<4; j++){

     M[i][j] = 0;
     U[i][j] = -1;
   }
 }

 fp = fopen("calabiyau", "w");
 /**************************************
  Let m[3] run from 1 to 2
  Let m[2] run from 2 to 4.
  Let m[1] run from 2 to 10.
  In each case m[i] should equal M[i,i]
 **************************************/
 for (m[3]=1; m[3]<3; m[3]++){
   M[3][3] = m[3];
   for(m[2]=2; m[2]<5; m[2]++){
     M[2][2]=m[2];
     for(m[1]=2; m[1]<11; m[1]++){
       M[1][1] = m[1];

     
       /* at this point we have chosen a matrix M */
       /* now we need to adjust the entries of U  */

       for(z0=0;z0<4; z0++){
         for(z1=0;z1<4; z1++){
           for(z2=0; z2<4; z2++){
             for(z3=0; z3<4; z3++){
               for(i=0; i<4; i++){ for(j=0; j<4; j++){ U[i][j] = -1; }}
               U[0][z0] = 0;
               U[1][z1] = 0;
               U[2][z2] = 0;
               U[3][z3] = 0;
             /******************************************
              So now we have a choice of M and a choice
              of zeros in each row.  Now the matrix we
              want is M + U.  Save this  as the matrix C.
              Also save a copy as R.
              ******************************************/
              for(i=0; i<4; i++){
              for(j=0; j < 4; j++){
                   C[i][j] = M[i][j] + U[i][j];
                   R[i][j] = C[i][j]; } }
             
	      /****************************************************
               Now that we have a 4x4 matrix chosen we need to 
               compute the minors along the first row.
               Let S hold the 3x3 submatrix and B[0]..B[3] will be 
               the four subdeterminants.
	      ****************************************************/
	      allzero = 0; /* assume that all a[i]'s are not 0 */
               for(i=1; i<4; i++){
                   for(j=1; j<4; j++){
                        S[i-1][j-1] = R[i][j];
                   }
               }
               B[0] = det3(S);

               /* Replace column 1 of S with col1,rows1,2,3 of R */
        
               for(i=1; i<4; i++){
		 S[i-1][0] = R[i][0];
               }
               B[1] = -1*det3(S);
    
	       /* Replace col 1 of S by col 1, rows 1,2,3 of R */

               for(i=1; i<4; i++){
		 S[i-1][1] = R[i][1];
               }
               B[2] = det3(S);

               /* Replace col 2 of S by col 2, rows 1,2,3 of R */
               
               for(i=1; i<4; i++){
		 S[i-1][2] = R[i][2];
               }
               B[3] = -1*det3(S);
           
               /* Now put B[0]..B[3] in increasing order.
                  save as A[0],..A[3]                     */
               for(i=1;i<4;i++){
		 if(B[0]>B[i]){
		   temp = B[0];
	           B[0] = B[i];
                   B[i] = temp;
                 }
	       }
               for(i=2;i<4;i++){
		 if(B[1]>B[i]){
                   temp = B[1];
		   B[1] = B[i];
                   B[i] = temp;
                 }
               }
              
	       if(B[2]>B[3]){
		  temp = B[2];
                  B[2] = B[3];
                  B[3] = temp;
               }
               nogood = 0;
               for(i=0;i<4;i++)
                 if(B[i]<=0)
                   nogood = 1;
               if(B[0] == 0 && B[1] == 0 && B[2] == 0 && B[3] == 0){
                 allzero = 1;
                 nogood = 0;
	       }

               if(nogood == 0){

               if(allzero == 1){
                 for(i=0;i<4;i++){
                   for(j=0;j<4;j++)
                     printf("%3d",C[i][j]);
	           printf("\n");
	         }
                 printf("[%3d %3d %3d %3d]\n\n",B[0],B[1],B[2],B[3]);
      	       }
               
               else{
                 t1 = gcd2(B[0],B[1]);
                 t2 = gcd2(t1,B[2]);
                 t3 = gcd2(t2,B[3]);
                 for(i=0;i<4;i++)
                   A[i] = B[i]/t3;
                 fprintf(fp,"%3d %3d %3d %3d\n",A[0],A[1],A[2],A[3]);
               }
	       } /* end of if nogood is false */

             }/* end of z3 loop */
           }/* end of z2 loop */
         }/* end of z1 loop */
       }/* end of z0 loop */

     } /* end of m[1] loop */
   } /* end of m[2] loop */
 } /* end of m[3] loop */
fclose(fp);

}/* end of main */

















