/*********************************************************************
  ckqsonevec.c

  Read in a 5-tuple a[0], a[1], a[2], a[3],a[4] from the file 
  "final5tuples".   So now a[0] <= a[1] <= a[2] <= a[3] <= a[4]
  and they are in lexicographical order, with no repetitions.
  Now I want to check the following conditions:

  Condition 1:

  For any 3 a[i],a[j],a[k] of the current 5-tuple check that
          d = a[0] + a[1] + a[2] + a[3] + a[4] - 1
  can be written as a linear combination
          d = b_i*a[i] + b_j*a[j] + b_k*a[k]
  where the integers b are greater than or equal to 0.

  

  Condition 2:

  Look at any a[i] and a[j].
  Either
    (i)  d is an integer linear combination of a[i], a[j]
  or
    (ii) There is a k and an l so that
      
           b_i*a[i] + b_j*a[j] = d - a[k]
           c_i*a[i] + b_j*a[j] = d - a[l]
         
         where a[l] and a[k] are any two of the remaining 
         three a's.                    

  If these are satisfied then write to the file "qsOK"
**********************************************************************/

#include<stdio.h>
#include<stdlib.h>

main()
{

   int a[5];
   int i,j,k,l;
   int keep, end;
   int d, try;
   int lincomb;
   int tripleOK, pairOK, keeptrying;
   int b1,b2,b3;
   int max1,max2,max3;
   int try1, try2;

   FILE *fp;
   FILE *nfp;

  
   a[0] = 407;
   a[1] = 547;
   a[2] = 5311;
   a[3] = 12528;
   a[4] = 18792;
 

    keep = 1;  /* assume this 5-tuple will work out */
    i = 0;
    j = i+1;
    k = j+1;
    d = a[0] + a[1] + a[2] + a[3] + a[4] - 1;
    printf("d = %6d\n", d);


    while(keep==1 && i<3){
      while(keep==1 && j <4){
	while(keep==1 && k < 5){

           tripleOK = 0; /* assume this triple will fail */
           /* test this triple */
           max1 = 1 + d/a[i]; 
           max2 = 1 + d/a[j]; 
           max3 = 1 + d/a[k];
           printf("i = %3d, j = %3d, k = %3d\n", i,j,k);
           printf("max1 = %3d, max2 = %3d, max3 = %3d\n", max1,max2,max3);
               for(b1 = 0; b1 < max1; b1++){
	          for(b2 = 0; b2 < max2; b2++){
                      for(b3 = 0; b3 < max3; b3++){
                          if(d == b1*a[i]+b2*a[j]+b3*a[k])
                              tripleOK = 1;
                      }
		  }
	       } /* end of testing linear combs */

 /* if tripleOK = 0 at this point, then this 5-tuple
    has failed.  Otherwise, we should move on to the
    next choice of i,j,k.                             */

    keep = tripleOK;

    k++;
	} /* end of k loop */
       j++;
       k = j + 1;
      } /* end of j loop */
     i++;
     j = i + 1;
     k = j + 1;
    }
    /* That completes the condition 1 testing.  If keep = 0
       we are done and we should move on to the next 5-tuple.
       Otherwise, we must check condition 2.                  */

    if(keep == 1){
 
      /* Test condition 2 */
      i = 0;
      j = i+1;
      while(keep == 1 && i < 4){
 	while(keep == 1 && j < 5){
	  pairOK = 0;  /* assume this pair will fail. */
          /* test this pair */
          printf("i = %3d, j = %3d\n", i,j);
          max1 = 1 + d/a[i];
          max2 = 1 + d/a[j];
          for(b1 = 0; b1 < max1; b1++){
	    for(b2 = 0; b2<max2; b2++){
               if(d == b1*a[i]+b2*a[j])
                  pairOK = 1;
            }
          } /* end of testing if d is a l.c. of a[i] and a[j] */

          if(pairOK == 0){ /* if d is not a l.c. of a[i] and a[j]
                              we must check if d-a[k] and d-a[l]
                              are, for some k, l                */
            for(k=0; k<4; k++){
              for(l=k+1; l<5; l++){
                  try1 = d - a[k];
                  try2 = d - a[l];
                  keeptrying = 0;   /* tells us if we need to look for l */
                  max1 = 1 + try1/a[i];
                  max2 = 1 + try1/a[j];
                  for(b1=0; b1<max1; b1++){
		    for(b2=0;b2<max2;b2++){
                      if(try1 == b1*a[i] + b2*a[j])
                         keeptrying = 1;
                    }
                  }/* end of checking if d-a[k] is a l.c of a[i],a[j] */
                  /* if they were then we must check d - a[l]         */
		  if(keeptrying == 1){
                    max1 = 1 + try2/a[i];
                    max2 = 1 + try2/a[j];
                    for(b1=0; b1<max1; b1++){
		    for(b2=0;b2<max2;b2++){
                      if(try2 == b1*a[i] + b2*a[j])
                         pairOK = 1;
                    }
		    }/* end of checking d - a[l] */   
		  } /* end of if keeptrying = 1 */
	      } /* end of l loop.  */
	   } /* end of k loop.  */
	  } /* end of checking if there is a k,l so that
               d-a[k] and d-a[l] are a linear combination
               of a[i] and a[j].                             */

   /*  At this point we have finished checking the current pair a[i],a[j].
       If it was all right then pairOK = 1 and we should check the next
       pair.  If pairOK = 0, then this 5-tuple should be thrown out.      */
       
       keep = pairOK;
       j++;
	}
       i++;
       j = i + 1;
      }   		

    }
  if(keep == 1)
     printf( "%5d %5d %5d %5d %5d\n", a[0],a[1],a[2],a[3],a[4]);
  


 
  
} /* end of main */






