/***********************************************************************
                 
determinant.c (June 7, 2000) 

We want to study the problem of solving the system below, later 
referred to as system (1):

   [ m0 + u00    u01       u02         u03      u04] [a0]      [-1] 
   [  u10      m1 + u11    u12         u13      u14] [a1]  =   [-1]
   [  u20        u21      m2 + u22     u23      u24] [a2]      [-1]
   [  u30        u31       u32      m3 + u33    u34] [a3]      [-1]
   [  u40        u41       u42         u43   m4+u44] [a4]      [-1]  
Most of the uij are equal to -1, but in each row we choose
one uij = 0.    
Further restrictions:  m0 is unknown
                       m1 is unknown      (compare to easycramer.c)
                       m2 = 3,4,...,16
                       m3 = 2,3,...,6
                       m4 = 1,2,3

The coefficient matrix of system (1) will be denoted as C in the
comments below.  As usual, we omit the unrealistic case where
C[4][4] = 0.

The integers m0 and m1 are unknown.  The determinant of C is 
a quadratic function of m0 and m1 whose coefficients we compute below:

   detC = c2*m0*m1 + c0*m0 + c1*m1 + c3

If it happens that c2 = 0, we print to the file "zerocorner".
Otherwise we print to the file "detcoefficients".

When we solve (1) by Cramer's rule we need also d0, the determinant
of the matrix obtained by replacing col 0 by -1's.  This will be
a linear function of m1, which we denote as
 
       d0 = a0*m1 + b0

We also need d1 = a1*m0 + b1,  obtained by replacing col 1 of C by -1's.  
Since a0 = d0/det C must be a positive integer and a1 = d1/det C must
be a positive integer, our hope is that we get bounds on the possible
values for m0 and for m1.

The coefficients in det C and d0 and d1 are computed by this program
and sent to the file "detcoefficients" to be processed further.
Each line of "detcoefficients" contains 8 coefficients

             a0, b0, a1, b1, c2, c0, c1, c3
***************************************************************************/

#include<stdio.h>
#include<stdlib.h>

main()
{
 int i,j;
 int M[5][5];        /* diagonal matrix with m_i = M[i][i]   */
 int m[5];    
 
 int U[5][5];            /* matrix whose entries are mostly -1   */
 int z0, z1, z2, z3, z4; /* where to put 0 in ith row of U       */
                         /* if j(i) = k then zi = k              */
 int C[5][5];        /* the coefficient matrix of system (1) */
 int R[5][5];        /* a copy of C for row reduction        */
 int nocondition;    /* if det C does not depend on m0       */

 int D,c0,c1,c2,c3,a0,b0,a1,b1;         /* det C              */
 int k;
 int d0, d1, d2, d3, d4; /* det's from Cramer's rule             */
 int S[4][4];            /* a 4x4 matrix, usually a subdet of di */
 int T[4][4];            /* a 4x4 matrix for testing */
 int s[3][3];
 int signchange;     /* did we switch two rows in reduction? */
 
 int GiveUp;         /* Is there a problem with this 4tuple? */
 
 int alpha[5];       /* Series case (a = 0, d0 nonzero)      */
 int shift[5];       /*                                      */
 int multiple;       /* is shift a multiple of (d1,d2,d3)?   */


 int A[5];           /* candidate 5-tuple                    */
 int B[5];           /* same as A, but throw away signs      */

 int t1, t2, t3, t4, t5; /* gcd of three of the A[i]'s           */

 int absb, L;        /* how far do we let m0 run?            */

 int temp;           /* used for sorting --- write the ai's  */
                     /* in increasing order                  */


  int gcd2(int a, int b){
      int r;
     
      if (a > b){
      r = a;
      a = b;
      b = r;
      }

      r = b%a;
      while (r > 0){
          b = a;
          a = r;
          r = b%a;
      }
      return a;
  }

  int gcd3(int a, int b, int c){
      return gcd2( gcd2(a,b), c);
  }

int det3( int S[3][3] ){
     int s1,s2,s3;
     s1 = S[0][0]*(S[1][1]*S[2][2] - S[1][2]*S[2][1]);
     s2 = S[1][0]*(S[0][2]*S[2][1] - S[0][1]*S[2][2]);
     s3 = S[2][0]*(S[0][1]*S[1][2] - S[0][2]*S[1][1]);
     
     return s1+s2+s3;
 }

 int det4(int S[4][4]){
     int i,j;
     int d;
     int temp[3][3];

     for(i=1;i<4;i++){              /* let temp hold rows 1 to 3 */
       for(j=1;j<4;j++){            /* and col1,col2,col3 of the */
         temp[i-1][j-1] = S[i][j];  /* 4x4 matrix S              */
       }
     }
     d = S[0][0]*det3(temp);

     for(i=1;i<4;i++)               /* Now we want col0,col2,col3 */
        temp[i-1][0] = S[i][0];
     d = d - S[0][1]*det3(temp);
 
     for(i=1;i<4;i++)               /* Now col0,col1,col3         */
        temp[i-1][1] = S[i][1];
     d = d +S[0][2]*det3(temp);

     for(i=1;i<4;i++)               /* and now col0,col1,col2    */
        temp[i-1][2] = S[i][2];
     d = d - S[0][3]*det3(temp);

     return d;
 }

FILE *fp;
FILE *mfp;
FILE *nfp;
FILE *kfp;


/******************************************
 Initialize the matrix M:
*******************************************/

 for(i = 0; i<5; i++){
   for(j=0; j<5; j++){
     M[i][j] = 0;
   }
 }

 /*********************************************************
  We have a coefficient matrix C with two unknown entries,
  m0 and m1.  We need expressions for the determinant of 
  C as a function of these unknowns.  We need two other
  determinants as well.  Replace col 0 of C with -1's and
  take the determinant.  Call this d0 --- it is a linear 
  function of m1.  Finally, replace col 1 of the original
  matrix C by -1's and take this determinant.  Call it 
  d1 --- it is a linear function of m0.
  ********************************************************/

 fp = fopen("detcoefficients", "w");
 nfp = fopen("zerocorner", "w");

 /**************************************
  Let m[4] run from 1 to 3
  Let m[3] run from 2 to 6
  Let m[2] run from 3 to 16.
  In each case m[i] should equal M[i,i]
 **************************************/
 for(m[4]=1;m[4]<4;m[4]++){ /* should run < 4 in final version */
   M[4][4] = m[4];
   for (m[3]=2; m[3]<7; m[3]++){ /* should run < 7 in final version */
    M[3][3] = m[3];
    for(m[2]=3; m[2]<17; m[2]++){ /* should run < 17 in final version */
      M[2][2]=m[2];
      


       /* at this point we have chosen a matrix M  */
       /* now we need to define the entries of U   */
       /* Start by defining U to have all entries  */
       /* equal to -1.  Then choose one entry in   */
       /* each row to be 0.                        */  

       for(z0=0;z0<5; z0++){
         for(z1=0;z1<5; z1++){
           for(z2=0; z2<5; z2++){
             for(z3=0; z3<5; z3++){
               for(z4=0; z4<5; z4++){
               for(i=0; i<5; i++){ 
                  for(j=0; j<5; j++) {
                      U[i][j] = -1; 
                  }
               }  /* Reinitialize U to be all -1's */

               U[0][z0] = 0;  /* put a 0 in the 0th row */
               U[1][z1] = 0;  /*                1st     */
               U[2][z2] = 0;  /*                2nd     */
               U[3][z3] = 0;  /*                3rd     */
               U[4][z4] = 0;  /* put a 0 in the 4th row */

               /*****************************************
              Now the coefficient matrix C of the system
              (1) can be formed by adding M and U.
              Save a copy as R.  Keep in mind that the 0,0
              entry and the 1,1 entry do not reflect the
              unknown values for m0 and m1.
	     *********************************************/            

              for(i=0; i<5; i++){
                for(j=0; j < 5; j++){
                    C[i][j] = M[i][j] + U[i][j];
                    R[i][j] = C[i][j]; 
                }
              }
	      
              if(C[4][4]!=0){ /* can't occur in a realistic
                                 solution to (1) so exclude */
              /*********************************************
                 First compute d0 = a0*m1 + b0.  
                 Replace column 0 of R by -1's. 
                 Do column operations to get 0's 
                 in row 1 and columns 2,3,4.
	      ***********************************************/
	      for(i=0;i<5;i++)
                 R[i][0] = -1;
              for(j=2;j<5;j++){
                k = R[1][j];
		for(i=0;i<5;i++){
                   R[i][j] = R[i][j] + k*R[i][0];
                }
              }

	      /***************************************************
                Now R is of the form
                   [ -1   u01   *   *  *]
                   [ -1   u11   0   0  0]
                   [ -1   u21   *   *  *]
                   [ -1   u31   *   *  *]
                   [ -1   u41   *   *  *]
                Expand the determinant on row 1, keeping in
                mind that the 1,1 entry should really be
                m1 + u11.

                d0 = a0*m1 + b0 where a0 = detT and 
                                      b0 = U[1][1]*detT -U[1][0]*det S
                T is formed by rows 0,2,3,4 and cols 0,2,3,4 of R
                S is formed by rows 0,2,3,4 and cols 1,2,3,4 of R
	      *****************************************************/
		 for(j=1;j<5;j++){
                   S[0][j-1] = R[0][j];
                   T[0][j-1] = R[0][j];
                 }
                for(i=2;i<5;i++)
		  for(j=1;j<5;j++){
                      S[i-1][j-1] = R[i][j];
                      T[i-1][j-1] = R[i][j];
                  }
                T[0][0] = R[0][0];
                for(i=2;i<5;i++)
                  T[i-1][0]=R[i][0];
 
                a0 = det4(T);
                b0 = -1*R[1][0]*det4(S) + R[1][1]*a0;                

		/*************************************************
                 Next reinitialize R to match the original
                 coefficient matrix C.  Replace col 1 by -1's
                 and compute d1 = a1*m0 + b1.
		*************************************************/

                for(i=0;i<5;i++)
                   for(j=0;j<5;j++)
                      R[i][j] = C[i][j];
             
                for(i=0;i<5;i++)
                   R[i][1] = -1;
                 
                   
              for(j=2;j<5;j++){
                k = R[0][j];
		for(i=0;i<5;i++){
                   R[i][j] = R[i][j] + k*R[i][1];
                }
              }
    
                /***************************************************
                  Now R is of the form:
                   [ m0 + u00  -1  0   0   0 ]
                   [   u10     -1  *   *   * ]
                   [   u20     -1  *   *   * ]
                   [   u30     -1  *   *   * ]
                   [   u40     -1  *   *   * ]
                 and we want its determinant.  Expand on row 0.
                 We get d1 = a1*m0 + b1 where a1 = det S
                 and b1 = u00*detS - u01*detT
                 S consists of rows 1,2,3,4 and cols 1,2,3,4 of R.
                 T consists of rows 1,2,3,4 and cols 0,2,3,4 of R.
		***************************************************/
                  for(i=1;i<5;i++)
		  for(j=1;j<5;j++){
                      S[i-1][j-1] = R[i][j];
                      T[i-1][j-1] = R[i][j];
                  }
                T[0][0] = R[1][0];
                for(i=2;i<5;i++)
                  T[i-1][0]=R[i][0];
 
                a1 = det4(S);
                b1 = -1*R[0][1]*det4(T) + R[0][0]*a1;

                /*****************************************************
                  Now we compute the determinant of the coefficient
                  matrix C.  It will be of the form
                   c2*m0*m1 + c0*m0 + c1*m1 + c3.
		**************************************************/
		c0 = 0;                   /* Initialize the c's */
                c1 = 0;
                c2 = 0;
                c3 = 0;
                for(i=0;i<5;i++)          /* Reinitialize R */
                  for(j=0;j<5;j++)
                    R[i][j] = C[i][j];

                /* Compute c2, the coefficient of m0,m1. This should
                   be simply the determinant of the 3x3 matrix made
                   from rows 2,3,4 and cols 2,3,4 of R.              */
             
                for(i=2; i<5; i++)
                   for(j=2;j<5;j++)
                      s[i-2][j-2] = R[i][j];
                c2 = det3(s);
                

                 
                     /******************************************
                     To compute the constant term c3 we simply 
                     set m0 = 0 and m1 = 0 and compute det C.

                     To compute the linear terms of det C:
                     If we set m0 = 0 and compute the determinant
                     we get c1*m1 + c3.  If we set m1 = 0 and
                     compute det C, then we get c0*m0 + c3.
                     Since I already have the constant term all
                     c0 will just be the determinant of the 
                     4x4 matrix obtained by deleting row 0 and
                     col 0 of C.  Similarly, the coefficient 
                     c1 of m1 will just be the determinant of
                     the 4x4 matrix obtained by deleting row 1
                     and column 1 of R.
                     *****************************************/
                for(i=1;i<5; i++)
                   for(j=1; j<5; j++)
                     S[i-1][j-1] = R[i][j];
                c3 = c3 + U[0][0]*det4(S); /* compute 0,0 minor of det C */
                c0 = det4(S);

                  /********************************************* 
                     Now as j runs from 0 to 3 we replace
                     column j of S by column j (rows 1,2,3,4)
                     of S.  This will give all the minors for
                     expanding det C along row 0 
		  ***********************************************/
                    signchange = 1;
		    for(j=0; j<4; j++){
                       signchange = -1*signchange;
                       for(i=0;i<4;i++)
                          S[i][j] = R[i+1][j];
                       c3 = c3 + signchange*U[0][j+1]*det4(S);
                    }
                

		    /****************************************
                    All that remains in c1.  Pull out the
                    4x4 matrix  obtained by deleting col1 and
                    row 1 of R and take the determinant.
                    *****************************************/
   
                    S[0][0] = R[0][0];
                    for(j=1;j<4;j++){
                       S[0][j] = R[0][j+1];
                       S[j][0] = R[j+1][0];
                    }
                    for(i=1;i<4;i++)
                       for(j=1;j<4;j++)
                          S[i][j] = R[i+1][j+1];

                    c1 = det4(S);
		   
   if(c2==0){
      fprintf(nfp, "%5d %5d ", a0,b0);
      fprintf(nfp, "%5d %5d ", a1,b1);
      fprintf(nfp, "%5d %5d %5d %5d\n", c2, c0, c1, c3);
      /********************************************************
      if(c0*c1<0)
         fprintf(nfp,"different signs\n");
      if(c0*c1==0)
         fprintf(nfp,"some zeros\n");
      if((c0==0) && (c1==0))
         fprintf(nfp,"both zero\n");
      fprintf(nfp, "Comes from the lower right corner =\n");
      for(i=2;i<5;i++){
	for(j=2;j<5;j++)
           fprintf(nfp, "%5d", C[i][j]);
        fprintf(nfp,"\n");
      }
      fprintf(nfp,"\n");  
      *********************************************************/
   }
   else{
      fprintf(fp, "%5d %5d ", a0,b0);
      fprintf(fp, "%5d %5d ", a1,b1);             
      fprintf(fp, "%5d %5d %5d %5d\n", c2, c0, c1, c3);
   }
		
                
                                     
	      } /* end of if C[4][4] is not zero */
	       }/* end of z4 loop */
	     }/* end of z3 loop */
           }/* end of z2 loop */
	 } /* end of z1 loop */
       }/* end of z0 loop */
	

   } /* end of m2 loop */
  } /* end of m3 loop */
 } /* end of m4 loop */
            
	       
fclose(fp);

} /* end of main */    



















