/***********************************************************************
                 
easycramer.c (June 26, 2000) 

We want to study the problem of solving the system below, later 
referred to as system (1):

   [ m0 + u00    u01       u02         u03      u04] [a0]      [-1] 
   [  u10      m1 + u11    u12         u13      u14] [a1]  =   [-1]
   [  u20        u21      m2 + u22     u23      u24] [a2]      [-1]
   [  u30        u31       u32      m3 + u33    u34] [a3]      [-1]
   [  u40        u41       u42         u43   m4+u44] [a4]      [-1]  
Most of the uij are equal to -1, but in each row we choose
one uij = 0.    
Further restrictions:  m0 is unknown
                       m1 = 3,4,...,83    NOTE:  m1 is not always
                       m2 = 3,4,...,16           bounded, but this
                       m3 = 2,3,...,6            is dealt with in
                       m4 = 1,2,3                the series programs.

The coefficient matrix of system (1) will be denoted as C in the
comments below.

All the numbers a0, a1, a2, a3 and a4 are to be positive integers
and any four of them should be relatively prime.

We solve by Cramer's Rule...Let d_i denote the determinant
obtained by replacing column i of C by -1's.  The determinant
of C depends on m0 and we get things like a0 = d0/(a*m0 + b).
The fact that we want this to be a positive integer puts
restrictions on m0.  We then let m0 run through all possible
values and look for 5-tuples [a0,a1,a2,a3,a4] that satisfy the
system (1), are positive integers, and any four are relatively
prime.  These are then reordered to satisfy a0<=a1<=a2<=a3<=a4
and printed to the file "ordered5tuples".

This program deals only with the easy cases --- it does not cope
with series or with possible zero over zero expressions.

***************************************************************************/

#include<stdio.h>
#include<stdlib.h>

main()
{
 int i,j;
 int M[5][5];        /* diagonal matrix with m_i = M[i][i]   */
 int m[5];    
 
 int U[5][5];            /* matrix whose entries are mostly -1   */
 int z0, z1, z2, z3, z4; /* where to put 0 in ith row of U       */
                         /* if j(i) = k then zi = k              */
 int C[5][5];        /* the coefficient matrix of system (1) */
 int R[5][5];        /* a copy of C for row reduction        */
 int badmatrix;      /* don't let C[3][3] = 0                */

 int D, a, b;            /* det C = D = a*m0 + b                 */
 int d0, d1, d2, d3, d4; /* det's from Cramer's rule             */
 int S[4][4];            /* a 4x4 matrix, usually a subdet of di */
 int T[4][4];            /* a 4x4 matrix for testing */
 int signchange;     /* did we switch two rows in reduction? */
 
 int GiveUp;         /* Is there a problem with this 5tuple? */
 
 int A[5];           /* candidate 5-tuple                    */

 int t1, t2, t3, t4, t5; /* gcd of four of the A[i]'s           */

 int absb, L;        /* how far do we let m0 run?            */

 int temp;           /* used for sorting --- write the ai's  */
                     /* in increasing order                  */



  int gcd2(int a, int b){
      int r;
     
      if (a > b){
      r = a;
      a = b;
      b = r;
      }

      r = b%a;
      while (r > 0){
          b = a;
          a = r;
          r = b%a;
      }
      return a;
  }

  int gcd3(int a, int b, int c){
      return gcd2( gcd2(a,b), c);
  }

int det3( int S[3][3] ){
     int s1,s2,s3;
     s1 = S[0][0]*(S[1][1]*S[2][2] - S[1][2]*S[2][1]);
     s2 = S[1][0]*(S[0][2]*S[2][1] - S[0][1]*S[2][2]);
     s3 = S[2][0]*(S[0][1]*S[1][2] - S[0][2]*S[1][1]);
     
     return s1+s2+s3;
 }

 int det4(int S[4][4]){
     int i,j;
     int d;
     int temp[3][3];

     for(i=1;i<4;i++){              /* let temp hold rows 1 to 3 */
       for(j=1;j<4;j++){            /* and col1,col2,col3 of the */
         temp[i-1][j-1] = S[i][j];  /* 4x4 matrix S              */
       }
     }
     d = S[0][0]*det3(temp);

     for(i=1;i<4;i++)               /* Now we want col0,col2,col3 */
        temp[i-1][0] = S[i][0];
     d = d - S[0][1]*det3(temp);
 
     for(i=1;i<4;i++)               /* Now col0,col1,col3         */
        temp[i-1][1] = S[i][1];
     d = d +S[0][2]*det3(temp);

     for(i=1;i<4;i++)               /* and now col0,col1,col2    */
        temp[i-1][2] = S[i][2];
     d = d - S[0][3]*det3(temp);

     return d;
 }

FILE *fp;



/******************************************
 Initialize the matrix M:
*******************************************/

 for(i = 0; i<5; i++){
   for(j=0; j<5; j++){
     M[i][j] = 0;
   }
 }

 fp = fopen("ordered5tuples", "w");

 /**************************************
  Let m[4] run from 1 to 3
  Let m[3] run from 2 to 6
  Let m[2] run from 3 to 16.
  Let m[1] run from 3 to 42.
  In each case m[i] should equal M[i,i]
 **************************************/
 for(m[4]=1;m[4]<4;m[4]++){
   M[4][4] = m[4];
  for (m[3]=2; m[3]<7; m[3]++){
    M[3][3] = m[3];
    for(m[2]=3; m[2]<17; m[2]++){ /* should run < 17 in final version */
      M[2][2]=m[2];
      for(m[1]=3; m[1]<84; m[1]++){ 
        M[1][1] = m[1];


       /* at this point we have chosen a matrix M  */
       /* now we need to define the entries of U   */
       /* Start by defining U to have all entries  */
       /* equal to -1.  Then choose one entry in   */
       /* each row to be 0.                        */  

       for(z0=0;z0<5; z0++){
         for(z1=0;z1<5; z1++){
           for(z2=0; z2<5; z2++){
             for(z3=0; z3<5; z3++){
               for(z4=0; z4<5; z4++){
               for(i=0; i<5; i++){ 
                  for(j=0; j<5; j++) {
                      U[i][j] = -1; 
                  }
               }  /* Reinitialize U to be all -1's */

               U[0][z0] = 0;  /* put a 0 in the 0th row */
               U[1][z1] = 0;  /*                1st     */
               U[2][z2] = 0;  /*                2nd     */
               U[3][z3] = 0;  /*                3rd     */
               U[4][z4] = 0;  /* put a 0 in the 4th row */

               /*****************************************
              Now the coefficient matrix C of the system
              (1) can be formed by adding M and U.
              Save a copy as R
	     *********************************************/            

              for(i=0; i<5; i++){
                for(j=0; j < 5; j++){
                    C[i][j] = M[i][j] + U[i][j];
                    R[i][j] = C[i][j]; 
                }
              }
 
	      /* Not all the C's constructed this way are
                 realistic for our problem.  For example,
                 it never happens that C[4][4] = 0 although
                 the loop above will produce such C's.  We
                 don't want to process those cases any
                 further.                                */
if(C[4][4]==0)
  badmatrix = 1;
else
  badmatrix = 0;

 if(badmatrix == 0){
              /*******************************************

                Replace column 0 of R by -1's.  Row reduce
                to make col 0 look like (-1 0 0 0 0)^T.
                Set S to be cols 1 to 4 and rows 1 to 4 of
                this reduced matrix.  Then d0 = -1*det4(S)

	      ********************************************/

              for(i=0;i<5;i++)
                 R[i][0] = -1;
              for(i=1;i<5;i++)
                 for(j=0;j<5;j++)
                    R[i][j] = R[i][j] - R[0][j];
              for(i=1;i<5;i++)
                for(j=1;j<5;j++)
                    S[i-1][j-1] = R[i][j];
              d0 = -1*det4(S);   
                
              /******************************************

               Now compute D = a*m0 + b, the determinant
               of the system.  Reinitialize R to match C.
               Then we can via row ops get the first col
               of R to look like either (*,0,0,0,0) or
               (*,-1,0,0,0).
               
	      *******************************************/
              for(i=0;i<5;i++)
                 for(j=0;j<5;j++)
                    R[i][j] = C[i][j];

              signchange = 1; /* assume that we don't need to
                                 switch any rows               */
              if(R[1][0]==-1){
                if(R[2][0]==-1){
                  for(j=0; j<5; j++){
                     R[2][j]=R[2][j] - R[1][j];
                  }
		}
               if(R[3][0]==-1){
                  for(j=0; j<5; j++){
                     R[3][j] = R[3][j] - R[1][j];
                  }
	       }
               if(R[4][0]==-1){
		 for(j=0; j<5; j++){
                    R[4][j] = R[4][j] - R[1][j];
                 }
               }
}    /* end of the case where column 0 started as (*,-1,*,*,*) */


/* At this point column 0 looks like (*,-1,0,0,0)  or we started
   with (*,0,*,*,*) in column 0.  In that case, if we have (*,0,-1,*,*) 
   we need to switch row 1 and row 2 and make sure we have a 0 
   in rows 3 and 4                                              */
if(R[2][0]==-1){
   for(j=0; j<5; j++){
         R[1][j] = C[2][j];
         R[2][j] = C[1][j];
         signchange = -1;
   }
         if(R[3][0] ==-1)
           for(j=0;j<5;j++){
            R[3][j] = R[3][j] - R[1][j];
           }
         if(R[4][0] ==-1)
	   for(j=0;j<5;j++){
            R[4][j] = R[4][j] - R[1][j];
           }
} /* end of the case where we start with (*,0,-1,*,*)   */

/* Again at this point column 0 looks like (*,-1,0,0,0) or we
   started with (*,0,0,*,*) in column 0.  If in row 3 we have
   -1 then we need to switch rows 1 and 3.               */

if(R[3][0]==-1){
  for(j=0; j<5; j++){
     R[1][j] = C[3][j];
     R[3][j] = C[1][j];
     signchange = -1;
  }
  if(R[4][0] == -1)
    for(j=0;j<5;j++){
       R[4][j] = R[4][j] - R[1][j];
    } 
} /* end of the case where we start with (*,0,0,-1,*) */

 if(R[4][0] == -1){
   for(j=0; j<5; j++){
      R[1][j] = C[4][j];
      R[4][j] = C[1][j];
   }
   signchange = -1;
 }/* now column 0 of R looks like (*,0,0,0,0) or like (*,-1,0,0,0) */

/* Now pull out the 4x4 submatrix of this reduced matrix
   consisting of cols 1,2,3,4 and rows 1,2,3,4               
   Take the determinant and adjust the sign if necessary.
   det C will be of the form a*m_0 + b where m_0 is 
   unknown.                                              */

      for(i = 1; i<5; i++){
        for(j=1; j<5; j++){
            S[i-1][j-1]= R[i][j];
         }
      }

      a = signchange*det4(S);

/************************************************************ 
   Now det C is a linear function of m0.  The coefficient
   of m0 will simply be the a computed above.  The rest will
   be either U[0][0]*a in the case where the reduced matrix
   has (*,0,0,0,0) as its column 0 or in the case where
   column 0 is of the form (*,-1,0,0,0) it will be 
   U[0][0]*a + det(4x4 matrix obtained by deleting column 0
   and row 1 of the reduced matrix R).
   
*************************************************************/

    if(R[1][0] == 0){
       b = U[0][0]*a;
    }
    else {  
      for(j=1; j<5; j++){
        S[0][j-1] = R[0][j];
        S[1][j-1] = R[2][j];
        S[2][j-1] = R[3][j];
        S[3][j-1] = R[4][j];
       }
       b = U[0][0]*a + signchange*det4(S);
    }



/******************************************************************
 We will deal with the bad cases for the problem later.  Now we consider
 what happens when det C depends on m0 (that is, a is not 0) and
 d0 is not zero so we will get a nonzero value for a0.

 There will be a limit on the possible size of m0.  If d0 is positive,
 then look at 
          d0          d0          d0
      ----------, ---------,  ----------,........
        a + b       2a + b      3a + b
       (m0 = 1)    (m0 = 2)    (m0 = 3)
 and so on until the denominator gets larger than d0.  If any
 of these is a positive integer, then we have a candidate for 
 a0.  Set m0 to this value and then compute d_i/(m0*a + b).
 If these are also positive integers, then we have a candidate
 5-tuple [a0,a1,a2,a3,a4].  They may not be listed in increasing
 order and we still must check the rel. prime condition.  If 
 that is satisfied, these 5-tuples will be written to the file 
 "ordered5tuples" in increasing order.

Depending on whether a is positive or negative we get 

      d0 - b          -b           -b
    ---------- < m0 < ---     or   --- < m0 < d0 - b
        a              a            a

*******************************************************************/
 if(d0 != 0){
   if(a != 0){
   L = (d0 - b)/a;
   if ( -b/a > L) 
      L = -b/a;
   for( m[0] = 1; m[0] <= L; m[0]++){
          GiveUp = 0;
           for(i=0;i<5;i++)
	    A[i] = 1;            /* initialize A */
          D = a*m[0] + b;
          if(D==0){
            GiveUp = 1;  
          }
          if(D!=0){
             if(d0 % D != 0){
                 GiveUp = 1;
	      }
	     else{ A[0] = d0/D;
	           if (A[0]<1)
                    {  
                      GiveUp = 1;
	            }
	     }
/***************************************************************
if this value for m0 is working so far, if it gives a legal 
value for a0, then we should compute a1,a2,a3,a4.  We have to 
adjust the matrix C to reflect the current value of m0 
to compute the other determinants.
*****************************************************************/

          C[0][0] = m[0] + U[0][0]; 
         
           
         /**************************************
                   Compute d1
	 **************************************/
           for(i=0; i<5; i++)
           for(j=0; j<5; j++)
             R[i][j] = C[i][j];       /* Reinitialize R */
         
           for(i=0; i<5; i++)
             R[i][1] = -1;            /* replace col 1 by -1's */
           for(i = 1; i < 5; i++){
             for(j=0; j<5; j++){
               R[i][j] = R[i][j] - R[0][j];   /* row reduce */
             }
           }
           for(i=1; i<5; i++)                /* S = the 0,1 minor of R */
             S[i-1][0] = R[i][0];
           for(i=1; i<5; i++)
             for(j=2; j<5; j++)
             S[i-1][j-1] = R[i][j];
           d1 = det4(S);

           if(d1 % D != 0){
               GiveUp = 1;
             }
           else{ A[1] = d1/D;
	         if(A[1]<1){
                 GiveUp=1;
                 }
           }
           
        /**************************************
                   Compute d2
	 **************************************/
           for(i=0; i<5; i++)
             for(j=0; j<5; j++)
               R[i][j] = C[i][j];       /* Reinitialize R */
          
           for(i=0; i<5; i++)
             R[i][2] = -1;
           for(i = 1; i < 5; i++){
             for(j=0; j<5; j++){
                R[i][j] = R[i][j] - R[0][j];
             }
           }
           for(i=1; i<5; i++){
              for(j=0; j<2; j++)
                S[i-1][j] = R[i][j];
           S[i-1][2] = R[i][3];
           S[i-1][3] = R[i][4];
           }
           
           d2 = -1*det4(S);
           if(d2 % D != 0){
               GiveUp = 1;
               }
           else{ A[2] = d2/D;
	          if(A[2]<1){
                    GiveUp=1;
                  }
           }
           
         /**************************************
                   Compute d3
	 **************************************/
           for(i=0; i<5; i++)
             for(j=0; j<5; j++)
               R[i][j] = C[i][j];       /* Reinitialize R */
          
             for(i=0; i<5; i++)
                R[i][3] = -1;
             for(i = 1; i < 5; i++){
                for(j=0; j<5; j++){
                   R[i][j] = R[i][j] - R[0][j];
                }
             }
             for(i=1; i<5; i++){
               for(j=0; j<3; j++)
                 S[i-1][j] = R[i][j];
               S[i-1][3] = R[i][4]; 
	     }
           
             d3 = det4(S);
             if(d3 % D != 0){
               GiveUp = 1;
              }
             else{ A[3] = d3/D;
	           if(A[3]<1){
                      GiveUp=1;
                   }
             }

         /**************************************
                   Compute d4
	 **************************************/
           for(i=0; i<5; i++)
             for(j=0; j<5; j++)
               R[i][j] = C[i][j];       /* Reinitialize R */
          
           for(i=0; i<5; i++)
             R[i][4] = -1;
           for(i = 1; i < 5; i++){
             for(j=0; j<5; j++){
                R[i][j] = R[i][j] - R[0][j];
             }
           }
           for(i=1; i<5; i++){
              for(j=0; j<4; j++)
                S[i-1][j] = R[i][j];
           }
           
           d4 = -1*det4(S);
           if(d4 % D != 0){
               GiveUp = 1;
               }
           else{ A[4] = d4/D;
	          if(A[4]<1){
                    GiveUp=1;
                  }
           }

	   if(GiveUp == 0){
             /* test for relative primeness */
          t1 = gcd2(gcd3(A[0],A[1],A[2]), A[3]);
          t2 = gcd2(gcd3(A[0],A[1],A[2]), A[4]);
          t3 = gcd2(gcd3(A[0],A[1],A[3]), A[4]);
          t4 = gcd2(gcd3(A[0],A[2],A[3]), A[4]);
          t5 = gcd2(gcd3(A[1],A[2],A[3]), A[4]);
          if(t1*t2*t3*t4*t5!=1)
             GiveUp = 1;                 
	   }
            

/***************************************************************** 
  The 5-tuples produced so far will not be in increasing order.  
  Next, we reorder the A[i]'s to get 
                A[0] <= A[1] <= A[2] <= A[3] <= A[4]
  when we record them in the file "ordered5tuples.
*******************************************************************/

if(GiveUp==0){
               for(i=1; i<5; i++){
		 if(A[i] < A[0]){
                  temp = A[0];
                  A[0] = A[i];
                  A[i] = temp;
		 }
	       }
               for(i=2; i<5; i++){
                 if(A[i] < A[1]){
                   temp = A[1];
                   A[1] = A[i];
                   A[i] = temp;
                 }
               }
               for(i=3; i<5; i++){
		 if(A[i] < A[2]){
                    temp = A[2];
                    A[2] = A[i];
                    A[i] = temp;
                 }
               }
               if(A[4]<A[3]){
                  temp = A[3];
                  A[3] = A[4];
                  A[4] = temp;
               }
             }
	   
if(GiveUp == 0){
  fprintf(fp, "%5d %5d %5d %5d %5d\n", A[0], A[1], A[2], A[3], A[4]);
	   }           

      }/* end of if D is not zero */
    } /* end of m[0] for loop */

   } /* end of if a is not zero */
 } /* end of if d0 is not zero */
 } /* end of processing this C if C[4][4] is not zero */
              
	       }/* end of z4 loop */
	     }/* end of z3 loop */
           }/* end of z2 loop */
	 } /* end of z1 loop */
       }/* end of z0 loop */
	

     }/* end of m1 loop */
   } /* end of m2 loop */
  } /* end of m3 loop */
 } /* end of m4 loop */
            
	       
fclose(fp);

} /* end of main */    


















