/*********************************************************************
  qsseries.c (modified from quasismooth.c)

  Read in a 5-tuple a[0], a[1], a[2], a[3],a[4] and a primitive
  shift vector ps[1], ps[2], ps[3], ps[4] from the file 
 "SortedSeries", output from the file "SortSeries".  For the
  series of 5-tuples of the form
 
  For [A[0],A[1],A[2],A[3],A[4]] = 
   [a[0],a[1]+t*ps[1],a[2]+t*qs[2],a[3]+t*qs[3],a[4]+t*qs[4]]

  Now I want to check the following conditions:

  Condition 1:
  (Don't allow A[0])
  For any 3 A[i],A[j],A[k] of the current 5-tuple check that
          d = A[0] + A[1] + A[2] + A[3] + A[4] - 1
  can be written as a linear combination
          d = b_i*A[i] + b_j*A[j] + b_k*A[k]
  where the integers b are greater than or equal to 0.

  if this is satisfied write to a file QSseries.

  Make a file "48triples" that contains ps[1],ps[2],ps[3] as a row by itself
  (to be used in "rmSeriesTerms.c")

  Condition 2:

  Look at any A[i] and A[j].
  Either
    (i)  d is an integer linear combination of A[i], A[j]
  or
    (ii) There is a k and an l so that
      
           b_i*A[i] + b_j*A[j] = d - A[k]
           c_i*A[i] + b_j*A[j] = d - A[l]
         
         where A[l] and A[k] are any two of the remaining 
         three A's.                    

**********************************************************************/

#include<stdio.h>
#include<stdlib.h>

main()
{

   int a[5];
   int ps[5];
   int at[5];
   int t, tmax;
   int i,j,k,l;
   int keep, end;
   int d, try;
   int ds, da;
   int lincomb;
   int tripleOK, pairOK, keeptrying;
   int b1,b2,b3;
   int max1,max2,max3;
   int try1, try2;
   
   FILE *fp;
   FILE *nfp;
   FILE *mfp;
   FILE *kfp;
  
  /* read in the first series from the file "SortedSeries" */
  
   fp = fopen("SortedSeries", "r");
   nfp = fopen("QSseries", "w"); /* whole series is quasismooth */
   mfp = fopen("QScksmallt", "w");  /* just some terms at the beginning */
   kfp = fopen("48triples", "w");

  end=fscanf( fp, "%d %d %d %d %d %d %d %d %d %d", 
         &a[0],&a[1],&a[2],&a[3],&a[4],&ps[0],&ps[1],&ps[2],&ps[3],&ps[4]);
 
  while(end > 0){
    keep = 1;  /* assume this series will work out */

    i = 1; /* don't allow a[0] to be chosen */
    j = i+1;
    k = j+1;

    da = a[0] + a[1] + a[2] + a[3] + a[4] - 1;
    ds = ps[1] + ps[2] + ps[3] + ps[4];

    while(keep==1 && i<3){
      while(keep==1 && j <4){
	while(keep==1 && k < 5){

           tripleOK = 0; /* assume this triple will fail */
           /* test this triple */
           max1 = 1 + ds/ps[i]; 
           max2 = 1 + ds/ps[j]; 
           max3 = 1 + ds/ps[k];
            for(b1 = 0; b1 < max1; b1++){
	      for(b2 = 0; b2 < max2; b2++){
               for(b3 = 0; b3 < max3; b3++){
                 if(da == b1*a[i]+b2*a[j]+b3*a[k] && ds==b1*ps[i]+b2*ps[j]+b3*ps[k])
                   tripleOK = 1;
               }
	      }
	    }
 /* if tripleOK = 0 at this point, then this 5-tuple
    has failed.  Otherwise, we should move on to the
    next choice of i,j,k.                             */
    keep = tripleOK;
         k++;
	}
       j++;
       k = j + 1;
      }
     i++;
     j = i+1;
     k = j + 1;
    }
    /* That completes the condition 1 testing.  If keep = 0
       we are done and we should move on to the next 5-tuple.
       Otherwise, we must check condition 2.                  */

    if(keep == 1){
 
      /* Test condition 2 */
      i = 1;  /* don't allow a[0] to be chosen */
      j = i+1;
      while(keep == 1 && i < 4){
      	while(keep == 1 && j < 5){
	  pairOK = 0;  /* assume this pair will fail. */
          /* test this pair */
          max1 = 1 + ds/ps[i];
          max2 = 1 + ds/ps[j];
          for(b1 = 0; b1 < max1; b1++){
	    for(b2 = 0; b2<max2; b2++){
               if(da == b1*a[i]+b2*a[j] && ds == b1*ps[i] + b2*ps[j])
                  pairOK = 1;
            }
          } /* end of testing if d is a l.c. of a[i] and a[j] */

          if(pairOK == 0){ /* if d is not a l.c. of a[i] and a[j]
                              we must check if d-a[k] and d-a[l]
                              are, for some k, l                */
            for(k=0; k<4; k++){
              for(l=k+1; l<5; l++){
                  try1 = da - a[k];
                  try2 = da - a[l];
                  keeptrying = 0;   /* tells us if we need to look for l */
                  max1 = 1 + try1/a[i];
                  max2 = 1 + try1/a[j];
                  for(b1=0; b1<max1; b1++){
		    for(b2=0;b2<max2;b2++){
                      if(try1==b1*a[i]+b2*a[j] && ds-ps[k]==b1*ps[i]+b2*ps[j])
                         keeptrying = 1;
                    }
                  }/* end of checking if d-a[k] is a l.c of a[i],a[j] */
                  /* if they were then we must check d - a[l]         */
		  if(keeptrying == 1){
                    max1 = 1 + try2/a[i];
                    max2 = 1 + try2/a[j];
                    for(b1=0; b1<max1; b1++){
		    for(b2=0;b2<max2;b2++){
                      if(try2 == b1*a[i] + b2*a[j] && ds-ps[l]==b1*ps[i]+b2*ps[j])
                         pairOK = 1;
                    }
		    }/* end of checking d - a[l] */   
		  } /* end of if keeptrying = 1 */
	      } /* end of l loop.  */
	   } /* end of k loop.  */
	  } /* end of checking if there is a k,l so that
               d-a[k] and d-a[l] are a linear combination
               of a[i] and a[j].                             */

   /*  At this point we have finished checking the current pair a[i],a[j].
       If it was all right then pairOK = 1 and we should check the next
       pair.  If pairOK = 0, then this 5-tuple should be thrown out.      */
       
       keep = pairOK;
       j++;
	}
       i++;
       j = i+1;
      }   		

    }
    if(keep == 1){
     fprintf(nfp, "%5d %5d %5d %5d %5d\n",a[0],a[1],a[2],a[3],a[4]);
     fprintf(nfp, "%5d %5d %5d %5d %5d\n",ps[0],ps[1],ps[2],ps[3],ps[4]);
     fprintf(nfp,"\n");
     fprintf(kfp,"%5d %5d %5d\n", ps[1],ps[2],ps[3]);

    }
    else{
      tmax = ds*(4 + a[1]/ps[1] + a[2]/ps[2] + a[3]/ps[3] + a[4]/ps[4]) + a[0];
      t = 1;
      while(t <= tmax){
         for(i=0;i<5;i++)
             at[i] = a[i] + t*ps[i];
         fprintf(mfp,"%5d %5d %5d %5d %5d\n", at[0],at[1],at[2],at[3],at[4]);
         t++;
      }
    } /* end of else --- checking for small t */
    
end=fscanf( fp, "%d %d %d %d %d %d %d %d %d %d", 
         &a[0],&a[1],&a[2],&a[3],&a[4],&ps[0],&ps[1],&ps[2],&ps[3],&ps[4]);

 } /* end of loop for reading in 5-tuples */

 
   fclose( fp );
   fclose( nfp );
   fclose( mfp );
   fclose( kfp);

} /* end of main */






