/**********************************************************************************************
                 
series5.c (June 15, 2000) 
modified from easycramer.c

In this file we look at the case when det C does not
depend on m0, that is when a = 0.  (See the file
00test.c for the case when a=0 and d0 =0).  Here
we expect to find infinite series.

   We compute det C = a*m0 + b and focus on the case
   where a = 0.
   
   Now det C = b.  Compute d0.  If d0/b is not a positive
   integer, then give up.  Otherwise, compute d1,d2 and d3.  
   These depend on m0.  Let d_i = m0*alpha_i + beta_i.
   The beta[i] don't matter for us but the alpha_i do.

   For m0 between 1 and |b| we compute d1,d2,d3,d4.  If di/b
   is an integer for every i, then 

        a[i] = m0*alpha[i] + beta[i]
               ----------------------
                        b
   is an integer and when we increase m0 by |b| we will increase
   a[i] by (|b|*alpha[i]/b).  Let shift[i] denote this quantity.
 
   For m0 between 1 and |b|, find all 5-tuples 
        [a0,a1,a2,a3,a4] = [d0/b,d1/b,d2/b,d3/b,d4/b]
   such that d0/b will be a positive integer and so that any
   three are relatively prime.  Then

        [a0,a1,a2,a3] = [ a0, a1 + k*shift[1], a2 + k*shift[2],
                                    a3 + k*shift[3],a4 + k*shift[3]]
   will be another solution in the same series.  

   Here we also compute the common factor of the shift components
   and divide them out to get a primitive shift vector.  We write
   these to the screen so we can see what is going on --- in the
   form 

   A[0]              A[1]       A[2]          A[3]        A[4]
   primshift[0] primshift[1] primshift[2] primshift[3] primshift[4] commonfactor = 

   They also get written to the file series in this form
   and they get written to a file called "seriestoprocess" in the form

   A[0] A[1] A[2] A[3] A[4] shift[0] shift[1] shift[2] shift[3] shift[4] commonfactor
************************************************************************************************/

#include<stdio.h>
#include<stdlib.h>

main()
{
 int i,j;
 int M[5][5];        /* diagonal matrix with m_i = M[i][i]   */
 int m[5];    
 
 int U[5][5];            /* matrix whose entries are mostly -1   */
 int z0, z1, z2, z3, z4; /* where to put 0 in ith row of U       */
                         /* if j(i) = k then zi = k              */
 int C[5][5];        /* the coefficient matrix of system (1) */
 int R[5][5];        /* a copy of C for row reduction        */
 int badmatrix;      /* don't let C[3][3] = 0                */
 int nocondition;    /* if det C does not depend on m0       */
 int count;
 int shiftdivides;
 int D, a, b;            /* det C = D = a*m0 + b                 */
 int d0, d1, d2, d3, d4; /* det's from Cramer's rule             */
 int S[4][4];            /* a 4x4 matrix, usually a subdet of di */
 int T[4][4];            /* a 4x4 matrix for testing */
 int signchange;     /* did we switch two rows in reduction? */
 
 int GiveUp;         /* Is there a problem with this 4tuple? */
 
 int alpha[5];       /* Series case (a = 0, d0 nonzero)      */
 int shift[5];       /*                                      */
 int primshift[5];   /*                                      */
 int multiple;       /* is shift a multiple of (d1,d2,d3)?   */
 int commonfactor;


 int A[5];           /* candidate 5-tuple                    */
 int B[5];           /* same as A, but throw away signs      */

 int t1, t2, t3, t4, t5; /* gcd of three of the A[i]'s           */

 int absb, L;        /* how far do we let m0 run?            */

 int temp;           /* used for sorting --- write the ai's  */
                     /* in increasing order                  */


  int gcd2(int a, int b){
      int r;
     
      if (a > b){
      r = a;
      a = b;
      b = r;
      }

      r = b%a;
      while (r > 0){
          b = a;
          a = r;
          r = b%a;
      }
      return a;
  }

  int gcd3(int a, int b, int c){
      return gcd2( gcd2(a,b), c);
  }

int det3( int S[3][3] ){
     int s1,s2,s3;
     s1 = S[0][0]*(S[1][1]*S[2][2] - S[1][2]*S[2][1]);
     s2 = S[1][0]*(S[0][2]*S[2][1] - S[0][1]*S[2][2]);
     s3 = S[2][0]*(S[0][1]*S[1][2] - S[0][2]*S[1][1]);
     
     return s1+s2+s3;
 }

 int det4(int S[4][4]){
     int i,j;
     int d;
     int temp[3][3];

     for(i=1;i<4;i++){              /* let temp hold rows 1 to 3 */
       for(j=1;j<4;j++){            /* and col1,col2,col3 of the */
         temp[i-1][j-1] = S[i][j];  /* 4x4 matrix S              */
       }
     }
     d = S[0][0]*det3(temp);

     for(i=1;i<4;i++)               /* Now we want col0,col2,col3 */
        temp[i-1][0] = S[i][0];
     d = d - S[0][1]*det3(temp);
 
     for(i=1;i<4;i++)               /* Now col0,col1,col3         */
        temp[i-1][1] = S[i][1];
     d = d +S[0][2]*det3(temp);

     for(i=1;i<4;i++)               /* and now col0,col1,col2    */
        temp[i-1][2] = S[i][2];
     d = d - S[0][3]*det3(temp);

     return d;
 }

FILE *fp;
FILE *mfp;
FILE *nfp;
FILE *kfp;


/******************************************
 Initialize the matrix M:
*******************************************/

 for(i = 0; i<5; i++){
   for(j=0; j<5; j++){
     M[i][j] = 0;
   }
 }

 nfp = fopen("series","w");
 kfp = fopen("seriestoprocess","w");
 
 count = 0;
 /**************************************
  Let m[4] run from 1 to 3
  Let m[3] run from 2 to 6
  Let m[2] run from 3 to 16.
  Let m[1] run from 3 to 84.
  In each case m[i] should equal M[i,i]
 **************************************/
 for(m[4]=1;m[4]<4;m[4]++){
   M[4][4] = m[4];
  for (m[3]=2; m[3]<7; m[3]++){
    M[3][3] = m[3];
    for(m[2]=3; m[2]<17; m[2]++){ /* should run < 17 in final version */
      M[2][2]=m[2];
      for(m[1]=3; m[1]<84; m[1]++){ 
        M[1][1] = m[1];


       /* at this point we have chosen a matrix M  */
       /* now we need to define the entries of U   */
       /* Start by defining U to have all entries  */
       /* equal to -1.  Then choose one entry in   */
       /* each row to be 0.                        */  

       for(z0=0;z0<5; z0++){
         for(z1=0;z1<5; z1++){
           for(z2=0; z2<5; z2++){
             for(z3=0; z3<5; z3++){
               for(z4=0; z4<5; z4++){
               for(i=0; i<5; i++){ 
                  for(j=0; j<5; j++) {
                      U[i][j] = -1; 
                  }
               }  /* Reinitialize U to be all -1's */

               U[0][z0] = 0;  /* put a 0 in the 0th row */
               U[1][z1] = 0;  /*                1st     */
               U[2][z2] = 0;  /*                2nd     */
               U[3][z3] = 0;  /*                3rd     */
               U[4][z4] = 0;  /* put a 0 in the 4th row */

               /*****************************************
              Now the coefficient matrix C of the system
              (1) can be formed by adding M and U.
              Save a copy as R
	     *********************************************/            

              for(i=0; i<5; i++){
                for(j=0; j < 5; j++){
                    C[i][j] = M[i][j] + U[i][j];
                    R[i][j] = C[i][j]; 
                }
              }
          
	      if(C[4][4] != 0){ /* an unrealistic choice of C --- move on */
              /*******************************************

                Replace column 0 of R by -1's.  Row reduce
                to make col 0 look like (-1 0 0 0 0)^T.
                Set S to be cols 1 to 4 and rows 1 to 4 of
                this reduced matrix.  Then d0 = -1*det4(S)

	      ********************************************/

              for(i=0;i<5;i++)
                 R[i][0] = -1;
              for(i=1;i<5;i++)
                 for(j=0;j<5;j++)
                    R[i][j] = R[i][j] - R[0][j];
              for(i=1;i<5;i++)
                for(j=1;j<5;j++)
                    S[i-1][j-1] = R[i][j];
              d0 = -1*det4(S);   
                
              /******************************************

               Now compute D = a*m0 + b, the determinant
               of the system.  Reinitialize R to match C.
               Then we can via row ops get the first col
               of R to look like either (*,0,0,0,0) or
               (*,-1,0,0,0).
               
	      *******************************************/
              for(i=0;i<5;i++)
                 for(j=0;j<5;j++)
                    R[i][j] = C[i][j];

              signchange = 1; /* assume that we don't need to
                                 switch any rows               */
              if(R[1][0]==-1){
                if(R[2][0]==-1){
                  for(j=0; j<5; j++){
                     R[2][j]=R[2][j] - R[1][j];
                  }
		}
               if(R[3][0]==-1){
                  for(j=0; j<5; j++){
                     R[3][j] = R[3][j] - R[1][j];
                  }
	       }
               if(R[4][0]==-1){
		 for(j=0; j<5; j++){
                    R[4][j] = R[4][j] - R[1][j];
                 }
               }
}    /* end of the case where column 0 started as (*,-1,*,*,*) */


/* At this point column 0 looks like (*,-1,0,0,0)  or we started
   with (*,0,*,*,*) in column 0.  In that case, if we have (*,0,-1,*,*) 
   we need to switch row 1 and row 2 and make sure we have a 0 
   in rows 3 and 4                                              */
if(R[2][0]==-1){
   for(j=0; j<5; j++){
         R[1][j] = C[2][j];
         R[2][j] = C[1][j];
         signchange = -1;
   }
         if(R[3][0] ==-1)
           for(j=0;j<5;j++){
            R[3][j] = R[3][j] - R[1][j];
           }
         if(R[4][0] ==-1)
	   for(j=0;j<5;j++){
            R[4][j] = R[4][j] - R[1][j];
           }
} /* end of the case where we start with (*,0,-1,*,*)   */

/* Again at this point column 0 looks like (*,-1,0,0,0) or we
   started with (*,0,0,*,*) in column 0.  If in row 3 we have
   -1 then we need to switch rows 1 and 3.               */

if(R[3][0]==-1){
  for(j=0; j<5; j++){
     R[1][j] = C[3][j];
     R[3][j] = C[1][j];
     signchange = -1;
  }
  if(R[4][0] == -1)
    for(j=0;j<5;j++){
       R[4][j] = R[4][j] - R[1][j];
    } 
} /* end of the case where we start with (*,0,0,-1,*) */

 if(R[4][0] == -1){
   for(j=0; j<5; j++){
      R[1][j] = C[4][j];
      R[4][j] = C[1][j];
   }
   signchange = -1;
 }/* now column 0 of R looks like (*,0,0,0,0) or like (*,-1,0,0,0) */

/* Now pull out the 4x4 submatrix of this reduced matrix
   consisting of cols 1,2,3,4 and rows 1,2,3,4               
   Take the determinant and adjust the sign if necessary.
   det C will be of the form a*m_0 + b where m_0 is 
   unknown.                                              */

      for(i = 1; i<5; i++){
        for(j=1; j<5; j++){
            S[i-1][j-1]= R[i][j];
         }
      }

      a = signchange*det4(S);

/************************************************************ 
   Now det C is a linear function of m0.  The coefficient
   of m0 will simply be the a computed above.  The rest will
   be either U[0][0]*a in the case where the reduced matrix
   has (*,0,0,0,0) as its column 0 or in the case where
   column 0 is of the form (*,-1,0,0,0) it will be 
   U[0][0]*a + det(4x4 matrix obtained by deleting column 0
   and row 1 of the reduced matrix R).
   
*************************************************************/

    if(R[1][0] == 0){
       b = U[0][0]*a;
    }
    else {  
      for(j=1; j<5; j++){
        S[0][j-1] = R[0][j];
        S[1][j-1] = R[2][j];
        S[2][j-1] = R[3][j];
        S[3][j-1] = R[4][j];
       }
       b = U[0][0]*a + signchange*det4(S);
    }

  

    if(d0 != 0 && a == 0){ /* look for series */
    if(d0*b > 0 && d0%b==0){

       if(b<0)
         absb = -1*b;
       else
         absb = b;

       D = b;
       A[0] = d0/D;

       for(m[0]=1;m[0]<=absb;m[0]++){

         /* initialize A */
          
          for(i=1; i<5; i++)
             A[i] = 1;
    
          GiveUp = 0; /* assume that this will give a 5-tuple */
          
           C[0][0] = m[0] + U[0][0];    
   /*****************************************************************
       Compute alpha[1], shift[1] and d1
   ******************************************************************/
     
    for(i=0; i<5; i++)
           for(j=0; j<5; j++)
             R[i][j] = C[i][j];       /* Reinitialize R */
            
    for(i=0; i<5; i++)
       R[i][1] = -1;           /* Set col 1 to be all -1 */

           /* Now alpha[1] will be the determinant of the
              4x4 matrix obtained by deleting column 1 and
              row 0 of the current R                       */
           
           for(i=1; i<5; i++){
              for(j=1; j<5; j++){
                 S[i-1][j-1] = R[i][j];
              }
           }
           
           alpha[1] = det4(S);
           shift[1] = absb*alpha[1]/b;
     
    for(i = 1; i < 5; i++){ /* subtract row 0 from rows 1,2,3,4 */
       for(j=0; j<5; j++){
          R[i][j] = R[i][j] - R[0][j];
       }
    }
    for(i=1; i<5; i++)
        S[i-1][0] = R[i][0];
    for(i=1; i<5; i++)
        for(j=2; j<5; j++)
           S[i-1][j-1] = R[i][j];
    d1 = det4(S);           

    if(d1 == 0)
      GiveUp = 1;
    if(d1 % D != 0)
      GiveUp = 1;
    else
       A[1] = d1/D;
    
  
  /*********************************************************
        Compute alpha[2], d2 and shift[2]
   **********************************************************/ 
     for(i=0; i<5; i++)
           for(j=0; j<5; j++)
             R[i][j] = C[i][j];       /* Reinitialize R */
         
    for(i=0; i<5; i++)
             R[i][2] = -1;           /* Set col 2 to be all -1 */

    for(i=1; i<5; i++){
      for(j=1; j<5; j++){
        S[i-1][j-1] = R[i][j];
      }
    }
    
     alpha[2] = det4(S);
     shift[2] = absb*alpha[2]/b;
       
     for(i = 1; i < 5; i++){ /* Subtract row 0 from rows 1,2,3,4 */
        for(j=0; j<5; j++){
            R[i][j] = R[i][j] - R[0][j];
        }
     }
     for(i=1; i<5; i++){  /* delete row 0 and col 2 */
        for(j=0; j<2; j++)
           S[i-1][j] = R[i][j];
        S[i-1][2] = R[i][3];
        S[i-1][3] = R[i][4];
     }
           
     d2 = -1*det4(S);
     if(d2 == 0)
       GiveUp = 1;
     if(d2 % D != 0)
       GiveUp = 1;
     else 
       A[2] = d2/D;
       

 /*********************************************************
        Compute alpha[3], d3 and shift[3]
   **********************************************************/ 
     for(i=0; i<5; i++)
           for(j=0; j<5; j++)
             R[i][j] = C[i][j];       /* Reinitialize R */
         
    for(i=0; i<5; i++)
             R[i][3] = -1;           /* Set col 3 to be all -1 */

    for(i=1; i<5; i++){
      for(j=1; j<5; j++){
        S[i-1][j-1] = R[i][j];
      }
    }
    
     alpha[3] = det4(S);
     shift[3] = absb*alpha[3]/b;
       
     for(i = 1; i < 5; i++){ /* Subtract row 0 from rows 1,2,3,4 */
        for(j=0; j<5; j++){
            R[i][j] = R[i][j] - R[0][j];
        }
     }
     for(i=1; i<5; i++){  /* delete row 0 and col 3 */
        for(j=0; j<3; j++)
           S[i-1][j] = R[i][j];
        S[i-1][3] = R[i][4];
     }
           
     d3 = det4(S);
     if(d3 == 0)
       GiveUp = 1;
     if(d3 % D != 0)
       GiveUp = 1;
     else 
       A[3] = d3/D;
      
 /*********************************************************
        Compute alpha[4], d4 and shift[4]
   **********************************************************/ 
     for(i=0; i<5; i++)
           for(j=0; j<5; j++)
             R[i][j] = C[i][j];       /* Reinitialize R */
         
    for(i=0; i<5; i++)
             R[i][4] = -1;           /* Set col 4 to be all -1 */

    for(i=1; i<5; i++){
      for(j=1; j<5; j++){
        S[i-1][j-1] = R[i][j];
      }
    }
    
     alpha[4] = det4(S);
     shift[4] = absb*alpha[4]/b;
       
     for(i = 1; i < 5; i++){ /* Subtract row 0 from rows 1,2,3,4 */
        for(j=0; j<5; j++){
            R[i][j] = R[i][j] - R[0][j];
        }
     }
     for(i=1; i<5; i++){  /* delete row 0 and col 4 */
        for(j=0; j<4; j++)
           S[i-1][j] = R[i][j];
     }
           
     d4 = -1*det4(S);
     if(d4 == 0)
       GiveUp = 1;
     if(d4 % D != 0)
       GiveUp = 1;
     else 
       A[4] = d4/D;

     commonfactor = gcd2(gcd3(shift[1],shift[2],shift[3]),shift[4]);

     if(GiveUp == 0){
        count++;
        for(i = 1; i<5; i++)
           primshift[i] = shift[i]/commonfactor;
        
        fprintf(nfp,"[%3d,%3d, %3d,%3d,%3d]\n",A[0],A[1],A[2],A[3],A[4]);
        fprintf(nfp,"[%3d %3d,%3d,%3d,%3d],  commonfactor = %3d\n\n\n",
             GiveUp,primshift[1],primshift[2],primshift[3],primshift[4], commonfactor);
        /* Now write to a file for further processing */
        fprintf(kfp,"%3d %3d %3d %3d %3d ",A[0],A[1],A[2],A[3],A[4]);
        fprintf(kfp,"%3d %3d %3d %3d %3d\n",shift[1],shift[2],shift[3],shift[4],commonfactor);
        /* print on screen so I can see what is happening. */
        printf("[%3d,%3d, %3d,%3d,%3d]\n",A[0],A[1],A[2],A[3],A[4]);
        printf("[%3d %3d,%3d,%3d,%3d],  commonfactor = %3d\n\n\n",
             GiveUp,primshift[1],primshift[2],primshift[3],primshift[4],commonfactor);
     }
  
       } /* end of the m[0] loop */


    } /* end of if a[0] is a positive integer */

    }/* end of if d0 is not zero and a is not zero */
	      } /* end of if C[4][4] is not zero */              
	       }/* end of z4 loop */
	     }/* end of z3 loop */
           }/* end of z2 loop */
	 } /* end of z1 loop */
       }/* end of z0 loop */
	

     }/* end of m1 loop */
   } /* end of m2 loop */
  } /* end of m3 loop */
 } /* end of m4 loop */
            
fclose(nfp);
fclose(kfp);

printf("We have a chance at %d series.\n", count);
} /* end of main */    


















