/*********************************************************************
  throwout.c

  Read in 8 numbers a0 b0 a1 b1 c2 c0 c1 c3 from the file 
  "zerocorner".   We have already checked that the expressions

   a0*m1 + b0, a1*m0 + b1, and c0*m0 + c1*m1 + c3 are dependent

  for any of the coefficients in "zerocorner".  In other words
       [ c0 c1 c3]
   det [ a1 0  b1] = a0a1c3 - a1b0c1 - a0b1c0 = 0.
       [ 0  a0 b0]

  In that case, look at

           a0*m1 + b0                    a1*m0 + b1
  a[0]= ------------------  and  a[1] = ------------------
        c0*m0 + c1*m1 + c3             c0*m0 + c1*m1 + c3

  
  c1*a1(a0m1 + b0) =       0*m0 + (a0a1c1)m1 + (a1b0c1)
  c0*a0(a1m0 + b1) = (a0a1c0)m0 +       0*m1 + (a0b1c0)

  and since a1b0c1 + a0b1c0 = a0a1c3 we have

   c1*a1(a0m1 + b0) +c0*a0(a1m0 + b1) = a0a1c0m0 + a0a1c1m1 + a0a1c3
                                      
  
  Therefore c1*a1*a[0] + c0*a0*a[1] = a0*a1.

  In this file we consider the equation

       c1*a1*u + c0*a0*v = a0*a1 
  
  If this equation has solutions (u,v) that are natural numbers
  between 1 and 8 then these coefficients may correspond to
  a viable 5-tuple and we will save them in the file "keep".
  Otherwise, these coefficients come from a system that will 
  never give the kind of 5-tuples we want.
 
  (We noticed in the output that we always got u = 1 and v = 1.
   This is verified by the program checkuv.c)
**********************************************************************/

#include<stdio.h>
#include<stdlib.h>


main()
{

   int a0,b0,a1,b1,c0,c1,c2,c3;
  
   int u,v;
   int end;
   int count;
   int keepcount;
        
   FILE *fp;
   FILE *nfp;
  
  
  /* read in the first 8tuple of coefficients from the file */
  
   fp = fopen("zerocorner", "r");
  nfp = fopen("keep", "w");

  end=fscanf( fp, "%d %d %d %d %d %d %d %d",&a0,&b0,&a1,&b1,&c2,&c0,&c1,&c3);  
  count = 0;
  keepcount = 0;

  while(end > 0){
   count ++;
  
     /***************************************************
        Now let u and v  run from 1 to 8. 
        Check whether c1a1u + c0a0v = a0a1.  If so
        write the 8-tuple to a file keep.
     ******************************************************/
    
     for(u=1; u<9; u++){
       for(v=1; v<9; v++){
         if(c1*a1*u + c0*a0*v == a0*a1 && c1*c0 > 0){
           keepcount ++;
       fprintf(nfp, "%3d %3d %3d %3d %3d %3d %3d %3d %3d\n",u,v,a0,b0,a1,b1,c0,c1,c3);
	 }
     }
     }

   end=fscanf( fp, "%d %d %d %d %d %d %d %d",&a0,&b0,&a1,&b1,&c2,&c0,&c1,&c3);

} /* end of while --- loop to read in coefficients */

   printf("processed %d 8-tuples.\n", count);
   printf("worked %d times\n", keepcount);
   fclose( fp );
   fclose( nfp );

 
  
} /* end of main */






