/***********************************************************

Reads in data from "seriestoprocess" a file created by
series5.c.  This data is in the form

a[0] a[1] a[2] a[3] a[4] shift[1] shift[2] shift[3] shift[4]

First we compute the common factor of the shift vector
components and divide it out to get a primitive shift vector
in increasing order and reorder the a's in the corresponding 
way.

Next we adjust a to be as small as possible by subtracting off a
multiple of the primitive shift vector.  The a[i]'s should be as
small as possible and still positive.  
This information is sent to the file "seriesnormalized" which
will be further processed.  

If it happens that the shift vector is a multiple of the a vector
then we don't get a series --- a + k*shift --- since unless k=0
the relatively prime condition will fail.  It could happen that
a itself is a new 5-tuple, so we write it to the file "serieskeq0".

CONSECUTIVE REPETITIONS in seriestoprocess are ignored.

**********************************************************************/

#include<stdio.h>
#include<stdlib.h>

main()
{

int a[5];
int b[5];
int cur[9];
int lw[9];
int k,d;
int tempa;
int ps[5];
int s[5];
int tempps;
int c;
int d2,d3,d4;

int end;
int i;
int junk;
int multiple;
int repeat;
int count;


 int gcd2(int a, int b){
      int r;
     
      if (a > b){
      r = a;
      a = b;
      b = r;
      }

      r = b%a;
      while (r > 0){
          b = a;
          a = r;
          r = b%a;
      }
      return a;
  }

  int gcd3(int a, int b, int c){
      return gcd2( gcd2(a,b), c);
  }


FILE *fp;
FILE *nfp;
FILE *kfp;

/* read in the first 5-tuple from the file */

fp = fopen("seriestoprocess", "r");
nfp = fopen("seriesnormalized", "w");
kfp = fopen("serieskeq0","w");

 ps[0] = 0; /* this will never change, we don't shift a[0] */

end = fscanf(fp, "%d %d %d %d %d %d %d %d %d %d",
                 &a[0],&a[1],&a[2],&a[3],&a[4],&s[1],&s[2],&s[3],&s[4],&junk);

c = gcd2(gcd3(s[1],s[2],s[3]),s[4]);

for(i=0;i<9;i++)
  lw[i] = 0;

 while(end>0){

   repeat = 1; /* assume we just read the same thing as last time. */

   for(i=1; i<5; i++)
       ps[i] = s[i]/c;


   /* Forget about the shift vectors.  Just work with ps instead.
      Reorder ps so that we have ps[1] <= ps[2] <= ps[3] <= ps[4].
      Reorder a's in the corresponding way.                        */
   
   for(i=2; i<5; i++){
     if(ps[i] < ps[1] ){
         tempps = ps[1];
         tempa = a[1];
         ps[1] = ps[i];
         a[1] = a[i];
         ps[i] = tempps;
          a[i] = tempa;
     }
   }
   for(i=3; i<5; i++){
     if(ps[i] < ps[2]){ /* need to switch ps[i] and ps[2] */
       tempps = ps[2];
       tempa = a[2];
       ps[2] = ps[i];
        a[2] = a[i];
       ps[i] = tempps;
        a[i] = tempa;
     }
   }
   for(i=4; i<5;i++){
     if(ps[i] < ps[3]){
       tempps = ps[3];
       tempa  = a[3];
       ps[3] = ps[i];
        a[3] =  a[i];
       ps[i] = tempps;
        a[i] = tempa;
     }
   }
   /*************************************************************
    Next we need to subtract a multiple of primshift to make
    a's as small as possible, but positive.
   *************************************************************/

    b[0] = a[0];
    cur[0] = b[0];

    if(a[1] % ps[1] != 0)
       k = a[1]/ps[1];
    else
       k = (a[1]/ps[1])-1;

    for(i=2;i<5;i++){
       if(a[i] % ps[i] != 0)
          d = a[i]/ps[i];
       else
          d = (a[i]/ps[i]) - 1;
       if(d < k)
          k = d;
    }

    for(i=1; i<5; i++){
        b[i] = a[i] - k*ps[i];
      cur[i] = b[i];
      cur[i+4] = ps[i];
    }

    for(i=0; i<9; i++)
       if(cur[i] != lw[i])
         repeat = 0;  

    if(repeat == 0){
        for(i=0;i<9;i++)
          lw[i] = cur[i];

        multiple = 0; /* assume that b and ps are different */

        if(a[1]*ps[2] - a[2]*ps[1] == 0)
           if(a[2]*ps[3] - a[3]*ps[2] == 0)
              if(a[3]*ps[4] - a[4]*ps[3] == 0)
                    multiple = 1;
        if(multiple == 0)
           fprintf(nfp,"%3d %3d %3d %3d %3d %3d %3d %3d %3d %3d\n",
                     b[0],b[1],b[2],b[3],b[4],ps[0],ps[1],ps[2],ps[3],ps[4]);
           else
             fprintf(kfp,"%3d %3d %3d %3d %3d\n", b[0],b[1],b[2],b[3],b[4]);
    }
   


    end = fscanf(fp,  "%d %d %d %d %d %d %d %d %d %d",
                 &a[0],&a[1],&a[2],&a[3],&a[4],&s[1],&s[2],&s[3],&s[4],&c);

 } /* end of while --- loop to read in coefficients */

 fclose(fp);
 fclose(nfp);
 fclose(kfp);

} /* end of main */
